! This is subroutine dmnfb and its dependencies in the PORT package, obtained
! from NETLIB 3/6/2012.
! D1MACH and I1MACH are replaced with Fortran 90 versions.

! 11/20/2017 changes to remove Fortran 2015 depreciated features
!              replace c and C in column 1 with ! for comments
!              replace CHARACTER*n with LEN=n
!              replace DABS with ABS and DMAX1 with MAX, DSQRT with SQRT; I
!                don't know if there are other specific intrinsic functions
!              replace labelled DO loops with DO ENDDO and GOTO ending label
!                with CYCLE
!              replace Hollerith edit descriptors with character strings
!              replace character in column 6 continuation with & at end
!              replace arithmetic if with equivalent set of IF ... GO TO
!              replace computed GO TO with a CASE block of GO TOs

! dmnfb finds a local minimum of a continuously differentiable function
! subject to simple upper and lower bound constraints.  I added it to PHAML
! for finding out if a point in 3D is on a surface patch of the domain
! defined by transfinite interpolation of 3 or 4 edges.

      SUBROUTINE DA7SST(IV, LIV, LV, V)
!
!  ***  ASSESS CANDIDATE STEP (***SOL VERSION 2.3)  ***
!
      INTEGER LIV, LV
      INTEGER IV(LIV)
      DOUBLE PRECISION V(LV)
!
!  ***  PURPOSE  ***
!
!        THIS SUBROUTINE IS CALLED BY AN UNCONSTRAINED MINIMIZATION
!     ROUTINE TO ASSESS THE NEXT CANDIDATE STEP.  IT MAY RECOMMEND ONE
!     OF SEVERAL COURSES OF ACTION, SUCH AS ACCEPTING THE STEP, RECOM-
!     PUTING IT USING THE SAME OR A NEW QUADRATIC MODEL, OR HALTING DUE
!     TO CONVERGENCE OR FALSE CONVERGENCE.  SEE THE RETURN CODE LISTING
!     BELOW.
!
!--------------------------  PARAMETER USAGE  --------------------------
!
!  IV (I/O) INTEGER PARAMETER AND SCRATCH VECTOR -- SEE DESCRIPTION
!             BELOW OF IV VALUES REFERENCED.
! LIV (IN)  LENGTH OF IV ARRAY.
!  LV (IN)  LENGTH OF V ARRAY.
!   V (I/O) REAL PARAMETER AND SCRATCH VECTOR -- SEE DESCRIPTION
!             BELOW OF V VALUES REFERENCED.
!
!  ***  IV VALUES REFERENCED  ***
!
!    IV(IRC) (I/O) ON INPUT FOR THE FIRST STEP TRIED IN A NEW ITERATION,
!             IV(IRC) SHOULD BE SET TO 3 OR 4 (THE VALUE TO WHICH IT IS
!             SET WHEN STEP IS DEFINITELY TO BE ACCEPTED).  ON INPUT
!             AFTER STEP HAS BEEN RECOMPUTED, IV(IRC) SHOULD BE
!             UNCHANGED SINCE THE PREVIOUS RETURN OF DA7SST.
!                ON OUTPUT, IV(IRC) IS A RETURN CODE HAVING ONE OF THE
!             FOLLOWING VALUES...
!                  1 = SWITCH MODELS OR TRY SMALLER STEP.
!                  2 = SWITCH MODELS OR ACCEPT STEP.
!                  3 = ACCEPT STEP AND DETERMINE V(RADFAC) BY GRADIENT
!                       TESTS.
!                  4 = ACCEPT STEP, V(RADFAC) HAS BEEN DETERMINED.
!                  5 = RECOMPUTE STEP (USING THE SAME MODEL).
!                  6 = RECOMPUTE STEP WITH RADIUS = V(LMAXS) BUT DO NOT
!                       EVALUATE THE OBJECTIVE FUNCTION.
!                  7 = X-CONVERGENCE (SEE V(XCTOL)).
!                  8 = RELATIVE FUNCTION CONVERGENCE (SEE V(RFCTOL)).
!                  9 = BOTH X- AND RELATIVE FUNCTION CONVERGENCE.
!                 10 = ABSOLUTE FUNCTION CONVERGENCE (SEE V(AFCTOL)).
!                 11 = SINGULAR CONVERGENCE (SEE V(LMAXS)).
!                 12 = FALSE CONVERGENCE (SEE V(XFTOL)).
!                 13 = IV(IRC) WAS OUT OF RANGE ON INPUT.
!             RETURN CODE I HAS PRECEDENCE OVER I+1 FOR I = 9, 10, 11.
! IV(MLSTGD) (I/O) SAVED VALUE OF IV(MODEL).
!  IV(MODEL) (I/O) ON INPUT, IV(MODEL) SHOULD BE AN INTEGER IDENTIFYING
!             THE CURRENT QUADRATIC MODEL OF THE OBJECTIVE FUNCTION.
!             IF A PREVIOUS STEP YIELDED A BETTER FUNCTION REDUCTION,
!             THEN IV(MODEL) WILL BE SET TO IV(MLSTGD) ON OUTPUT.
! IV(NFCALL) (IN)  INVOCATION COUNT FOR THE OBJECTIVE FUNCTION.
! IV(NFGCAL) (I/O) VALUE OF IV(NFCALL) AT STEP THAT GAVE THE BIGGEST
!             FUNCTION REDUCTION THIS ITERATION.  IV(NFGCAL) REMAINS
!             UNCHANGED UNTIL A FUNCTION REDUCTION IS OBTAINED.
! IV(RADINC) (I/O) THE NUMBER OF RADIUS INCREASES (OR MINUS THE NUMBER
!             OF DECREASES) SO FAR THIS ITERATION.
! IV(RESTOR) (OUT) SET TO 1 IF V(F) HAS BEEN RESTORED AND X SHOULD BE
!             RESTORED TO ITS INITIAL VALUE, TO 2 IF X SHOULD BE SAVED,
!             TO 3 IF X SHOULD BE RESTORED FROM THE SAVED VALUE, AND TO
!             0 OTHERWISE.
!  IV(STAGE) (I/O) COUNT OF THE NUMBER OF MODELS TRIED SO FAR IN THE
!             CURRENT ITERATION.
! IV(STGLIM) (IN)  MAXIMUM NUMBER OF MODELS TO CONSIDER.
! IV(SWITCH) (OUT) SET TO 0 UNLESS A NEW MODEL IS BEING TRIED AND IT
!             GIVES A SMALLER FUNCTION VALUE THAN THE PREVIOUS MODEL,
!             IN WHICH CASE DA7SST SETS IV(SWITCH) = 1.
! IV(TOOBIG) (I/O)  IS NONZERO ON INPUT IF STEP WAS TOO BIG (E.G., IF
!             IT WOULD CAUSE OVERFLOW).  IT IS SET TO 0 ON RETURN.
!   IV(XIRC) (I/O) VALUE THAT IV(IRC) WOULD HAVE IN THE ABSENCE OF
!             CONVERGENCE, FALSE CONVERGENCE, AND OVERSIZED STEPS.
!
!  ***  V VALUES REFERENCED  ***
!
! V(AFCTOL) (IN)  ABSOLUTE FUNCTION CONVERGENCE TOLERANCE.  IF THE
!             ABSOLUTE VALUE OF THE CURRENT FUNCTION VALUE V(F) IS LESS
!             THAN V(AFCTOL) AND DA7SST DOES NOT RETURN WITH
!             IV(IRC) = 11, THEN DA7SST RETURNS WITH IV(IRC) = 10.
! V(DECFAC) (IN)  FACTOR BY WHICH TO DECREASE RADIUS WHEN IV(TOOBIG) IS
!             NONZERO.
! V(DSTNRM) (IN)  THE 2-NORM OF D*STEP.
! V(DSTSAV) (I/O) VALUE OF V(DSTNRM) ON SAVED STEP.
!   V(DST0) (IN)  THE 2-NORM OF D TIMES THE NEWTON STEP (WHEN DEFINED,
!             I.E., FOR V(NREDUC) .GE. 0).
!      V(F) (I/O) ON BOTH INPUT AND OUTPUT, V(F) IS THE OBJECTIVE FUNC-
!             TION VALUE AT X.  IF X IS RESTORED TO A PREVIOUS VALUE,
!             THEN V(F) IS RESTORED TO THE CORRESPONDING VALUE.
!   V(FDIF) (OUT) THE FUNCTION REDUCTION V(F0) - V(F) (FOR THE OUTPUT
!             VALUE OF V(F) IF AN EARLIER STEP GAVE A BIGGER FUNCTION
!             DECREASE, AND FOR THE INPUT VALUE OF V(F) OTHERWISE).
! V(FLSTGD) (I/O) SAVED VALUE OF V(F).
!     V(F0) (IN)  OBJECTIVE FUNCTION VALUE AT START OF ITERATION.
! V(GTSLST) (I/O) VALUE OF V(GTSTEP) ON SAVED STEP.
! V(GTSTEP) (IN)  INNER PRODUCT BETWEEN STEP AND GRADIENT.
! V(INCFAC) (IN)  MINIMUM FACTOR BY WHICH TO INCREASE RADIUS.
!  V(LMAXS) (IN)  MAXIMUM REASONABLE STEP SIZE (AND INITIAL STEP BOUND).
!             IF THE ACTUAL FUNCTION DECREASE IS NO MORE THAN TWICE
!             WHAT WAS PREDICTED, IF A RETURN WITH IV(IRC) = 7, 8, OR 9
!             DOES NOT OCCUR, IF V(DSTNRM) .GT. V(LMAXS) OR THE CURRENT
!             STEP IS A NEWTON STEP, AND IF
!             V(PREDUC) .LE. V(SCTOL) * ABS(V(F0)), THEN DA7SST RETURNS
!             WITH IV(IRC) = 11.  IF SO DOING APPEARS WORTHWHILE, THEN
!            DA7SST REPEATS THIS TEST (DISALLOWING A FULL NEWTON STEP)
!             WITH V(PREDUC) COMPUTED FOR A STEP OF LENGTH V(LMAXS)
!             (BY A RETURN WITH IV(IRC) = 6).
! V(NREDUC) (I/O)  FUNCTION REDUCTION PREDICTED BY QUADRATIC MODEL FOR
!             NEWTON STEP.  IF DA7SST IS CALLED WITH IV(IRC) = 6, I.E.,
!             IF V(PREDUC) HAS BEEN COMPUTED WITH RADIUS = V(LMAXS) FOR
!             USE IN THE SINGULAR CONVERGENCE TEST, THEN V(NREDUC) IS
!             SET TO -V(PREDUC) BEFORE THE LATTER IS RESTORED.
! V(PLSTGD) (I/O) VALUE OF V(PREDUC) ON SAVED STEP.
! V(PREDUC) (I/O) FUNCTION REDUCTION PREDICTED BY QUADRATIC MODEL FOR
!             CURRENT STEP.
! V(RADFAC) (OUT) FACTOR TO BE USED IN DETERMINING THE NEW RADIUS,
!             WHICH SHOULD BE V(RADFAC)*DST, WHERE  DST  IS EITHER THE
!             OUTPUT VALUE OF V(DSTNRM) OR THE 2-NORM OF
!             DIAG(NEWD)*STEP  FOR THE OUTPUT VALUE OF STEP AND THE
!             UPDATED VERSION, NEWD, OF THE SCALE VECTOR D.  FOR
!             IV(IRC) = 3, V(RADFAC) = 1.0 IS RETURNED.
! V(RDFCMN) (IN)  MINIMUM VALUE FOR V(RADFAC) IN TERMS OF THE INPUT
!             VALUE OF V(DSTNRM) -- SUGGESTED VALUE = 0.1.
! V(RDFCMX) (IN)  MAXIMUM VALUE FOR V(RADFAC) -- SUGGESTED VALUE = 4.0.
!  V(RELDX) (IN) SCALED RELATIVE CHANGE IN X CAUSED BY STEP, COMPUTED
!             (E.G.) BY FUNCTION  DRLDST  AS
!                 MAX (D(I)*ABS(X(I)-X0(I)), 1 .LE. I .LE. P) /
!                    MAX (D(I)*(ABS(X(I))+ABS(X0(I))), 1 .LE. I .LE. P).
! V(RFCTOL) (IN)  RELATIVE FUNCTION CONVERGENCE TOLERANCE.  IF THE
!             ACTUAL FUNCTION REDUCTION IS AT MOST TWICE WHAT WAS PRE-
!             DICTED AND  V(NREDUC) .LE. V(RFCTOL)*ABS(V(F0)),  THEN
!            DA7SST RETURNS WITH IV(IRC) = 8 OR 9.
!  V(SCTOL) (IN)  SINGULAR CONVERGENCE TOLERANCE -- SEE V(LMAXS).
! V(STPPAR) (IN)  MARQUARDT PARAMETER -- 0 MEANS FULL NEWTON STEP.
! V(TUNER1) (IN)  TUNING CONSTANT USED TO DECIDE IF THE FUNCTION
!             REDUCTION WAS MUCH LESS THAN EXPECTED.  SUGGESTED
!             VALUE = 0.1.
! V(TUNER2) (IN)  TUNING CONSTANT USED TO DECIDE IF THE FUNCTION
!             REDUCTION WAS LARGE ENOUGH TO ACCEPT STEP.  SUGGESTED
!             VALUE = 10**-4.
! V(TUNER3) (IN)  TUNING CONSTANT USED TO DECIDE IF THE RADIUS
!             SHOULD BE INCREASED.  SUGGESTED VALUE = 0.75.
!  V(XCTOL) (IN)  X-CONVERGENCE CRITERION.  IF STEP IS A NEWTON STEP
!             (V(STPPAR) = 0) HAVING V(RELDX) .LE. V(XCTOL) AND GIVING
!             AT MOST TWICE THE PREDICTED FUNCTION DECREASE, THEN
!            DA7SST RETURNS IV(IRC) = 7 OR 9.
!  V(XFTOL) (IN)  FALSE CONVERGENCE TOLERANCE.  IF STEP GAVE NO OR ONLY
!             A SMALL FUNCTION DECREASE AND V(RELDX) .LE. V(XFTOL),
!             THEN DA7SST RETURNS WITH IV(IRC) = 12.
!
!-------------------------------  NOTES  -------------------------------
!
!  ***  APPLICATION AND USAGE RESTRICTIONS  ***
!
!        THIS ROUTINE IS CALLED AS PART OF THE NL2SOL (NONLINEAR
!     LEAST-SQUARES) PACKAGE.  IT MAY BE USED IN ANY UNCONSTRAINED
!     MINIMIZATION SOLVER THAT USES DOGLEG, GOLDFELD-QUANDT-TROTTER,
!     OR LEVENBERG-MARQUARDT STEPS.
!
!  ***  ALGORITHM NOTES  ***
!
!        SEE (1) FOR FURTHER DISCUSSION OF THE ASSESSING AND MODEL
!     SWITCHING STRATEGIES.  WHILE NL2SOL CONSIDERS ONLY TWO MODELS,
!    DA7SST IS DESIGNED TO HANDLE ANY NUMBER OF MODELS.
!
!  ***  USAGE NOTES  ***
!
!        ON THE FIRST CALL OF AN ITERATION, ONLY THE I/O VARIABLES
!     STEP, X, IV(IRC), IV(MODEL), V(F), V(DSTNRM), V(GTSTEP), AND
!     V(PREDUC) NEED HAVE BEEN INITIALIZED.  BETWEEN CALLS, NO I/O
!     VALUES EXCEPT STEP, X, IV(MODEL), V(F) AND THE STOPPING TOLER-
!     ANCES SHOULD BE CHANGED.
!        AFTER A RETURN FOR CONVERGENCE OR FALSE CONVERGENCE, ONE CAN
!     CHANGE THE STOPPING TOLERANCES AND CALL DA7SST AGAIN, IN WHICH
!     CASE THE STOPPING TESTS WILL BE REPEATED.
!
!  ***  REFERENCES  ***
!
!     (1) DENNIS, J.E., JR., GAY, D.M., AND WELSCH, R.E. (1981),
!        AN ADAPTIVE NONLINEAR LEAST-SQUARES ALGORITHM,
!        ACM TRANS. MATH. SOFTWARE, VOL. 7, NO. 3.
!
!     (2) POWELL, M.J.D. (1970)  A FORTRAN SUBROUTINE FOR SOLVING
!        SYSTEMS OF NONLINEAR ALGEBRAIC EQUATIONS, IN NUMERICAL
!        METHODS FOR NONLINEAR ALGEBRAIC EQUATIONS, EDITED BY
!        P. RABINOWITZ, GORDON AND BREACH, LONDON.
!
!  ***  HISTORY  ***
!
!        JOHN DENNIS DESIGNED MUCH OF THIS ROUTINE, STARTING WITH
!     IDEAS IN (2). ROY WELSCH SUGGESTED THE MODEL SWITCHING STRATEGY.
!        DAVID GAY AND STEPHEN PETERS CAST THIS SUBROUTINE INTO A MORE
!     PORTABLE FORM (WINTER 1977), AND DAVID GAY CAST IT INTO ITS
!     PRESENT FORM (FALL 1978), WITH MINOR CHANGES TO THE SINGULAR
!     CONVERGENCE TEST IN MAY, 1984 (TO DEAL WITH FULL NEWTON STEPS).
!
!  ***  GENERAL  ***
!
!     THIS SUBROUTINE WAS WRITTEN IN CONNECTION WITH RESEARCH
!     SUPPORTED BY THE NATIONAL SCIENCE FOUNDATION UNDER GRANTS
!     MCS-7600324, DCR75-10143, 76-14311DSS, MCS76-11989, AND
!     MCS-7906671.
!
!------------------------  EXTERNAL QUANTITIES  ------------------------
!
!  ***  NO EXTERNAL FUNCTIONS AND SUBROUTINES  ***
!
!--------------------------  LOCAL VARIABLES  --------------------------
!
      LOGICAL GOODX
      INTEGER I, NFC
      DOUBLE PRECISION EMAX, EMAXS, GTS, RFAC1, XMAX
      DOUBLE PRECISION HALF, ONE, ONEP2, TWO, ZERO
!
!  ***  SUBSCRIPTS FOR IV AND V  ***
!
      INTEGER AFCTOL, DECFAC, DSTNRM, DSTSAV, DST0, F, FDIF, FLSTGD, F0, &
              GTSLST, GTSTEP, INCFAC, IRC, LMAXS, MLSTGD, MODEL, NFCALL, &
              NFGCAL, NREDUC, PLSTGD, PREDUC, RADFAC, RADINC, RDFCMN, &
              RDFCMX, RELDX, RESTOR, RFCTOL, SCTOL, STAGE, STGLIM, &
              STPPAR, SWITCH, TOOBIG, TUNER1, TUNER2, TUNER3, XCTOL, &
              XFTOL, XIRC
!
!  ***  DATA INITIALIZATIONS  ***
!
!/6
!     DATA HALF/0.5D+0/, ONE/1.D+0/, ONEP2/1.2D+0/, TWO/2.D+0/,
!    1     ZERO/0.D+0/
!/7
      PARAMETER (HALF=0.5D+0, ONE=1.D+0, ONEP2=1.2D+0, TWO=2.D+0, &
                 ZERO=0.D+0)
!/
!
!/6
!     DATA IRC/29/, MLSTGD/32/, MODEL/5/, NFCALL/6/, NFGCAL/7/,
!    1     RADINC/8/, RESTOR/9/, STAGE/10/, STGLIM/11/, SWITCH/12/,
!    2     TOOBIG/2/, XIRC/13/
!/7
      PARAMETER (IRC=29, MLSTGD=32, MODEL=5, NFCALL=6, NFGCAL=7, &
                 RADINC=8, RESTOR=9, STAGE=10, STGLIM=11, SWITCH=12, &
                 TOOBIG=2, XIRC=13)
!/
!/6
!     DATA AFCTOL/31/, DECFAC/22/, DSTNRM/2/, DST0/3/, DSTSAV/18/,
!    1     F/10/, FDIF/11/, FLSTGD/12/, F0/13/, GTSLST/14/, GTSTEP/4/,
!    2     INCFAC/23/, LMAXS/36/, NREDUC/6/, PLSTGD/15/, PREDUC/7/,
!    3     RADFAC/16/, RDFCMN/24/, RDFCMX/25/, RELDX/17/, RFCTOL/32/,
!    4     SCTOL/37/, STPPAR/5/, TUNER1/26/, TUNER2/27/, TUNER3/28/,
!    5     XCTOL/33/, XFTOL/34/
!/7
      PARAMETER (AFCTOL=31, DECFAC=22, DSTNRM=2, DST0=3, DSTSAV=18, &
                 F=10, FDIF=11, FLSTGD=12, F0=13, GTSLST=14, GTSTEP=4, &
                 INCFAC=23, LMAXS=36, NREDUC=6, PLSTGD=15, PREDUC=7, &
                 RADFAC=16, RDFCMN=24, RDFCMX=25, RELDX=17, RFCTOL=32, &
                 SCTOL=37, STPPAR=5, TUNER1=26, TUNER2=27, TUNER3=28, &
                 XCTOL=33, XFTOL=34)
!/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      NFC = IV(NFCALL)
      IV(SWITCH) = 0
      IV(RESTOR) = 0
      RFAC1 = ONE
      GOODX = .TRUE.
      I = IV(IRC)
      IF (I .GE. 1 .AND. I .LE. 12) THEN
!                   GO TO (20,30,10,10,40,280,220,220,220,220,220,170), I
         SELECT CASE(I)
         CASE(1)
            GO TO 20
         CASE(2)
            GO TO 30
         CASE(3)
            GO TO 10
         CASE(4)
            GO TO 10
         CASE(5)
            GO TO 40
         CASE(6)
            GO TO 280
         CASE(7)
            GO TO 220
         CASE(8)
            GO TO 220
         CASE(9)
            GO TO 220
         CASE(10)
            GO TO 220
         CASE(11)
            GO TO 220
         CASE(12)
            GO TO 170
         END SELECT
      ENDIF
         IV(IRC) = 13
         GO TO 999
!
!  ***  INITIALIZE FOR NEW ITERATION  ***
!
 10   IV(STAGE) = 1
      IV(RADINC) = 0
      V(FLSTGD) = V(F0)
      IF (IV(TOOBIG) .EQ. 0) GO TO 110
         IV(STAGE) = -1
         IV(XIRC) = I
         GO TO 60
!
!  ***  STEP WAS RECOMPUTED WITH NEW MODEL OR SMALLER RADIUS  ***
!  ***  FIRST DECIDE WHICH  ***
!
 20   IF (IV(MODEL) .NE. IV(MLSTGD)) GO TO 30
!        ***  OLD MODEL RETAINED, SMALLER RADIUS TRIED  ***
!        ***  DO NOT CONSIDER ANY MORE NEW MODELS THIS ITERATION  ***
         IV(STAGE) = IV(STGLIM)
         IV(RADINC) = -1
         GO TO 110
!
!  ***  A NEW MODEL IS BEING TRIED.  DECIDE WHETHER TO KEEP IT.  ***
!
 30   IV(STAGE) = IV(STAGE) + 1
!
!     ***  NOW WE ADD THE POSSIBILITY THAT STEP WAS RECOMPUTED WITH  ***
!     ***  THE SAME MODEL, PERHAPS BECAUSE OF AN OVERSIZED STEP.     ***
!
 40   IF (IV(STAGE) .GT. 0) GO TO 50
!
!        ***  STEP WAS RECOMPUTED BECAUSE IT WAS TOO BIG.  ***
!
         IF (IV(TOOBIG) .NE. 0) GO TO 60
!
!        ***  RESTORE IV(STAGE) AND PICK UP WHERE WE LEFT OFF.  ***
!
         IV(STAGE) = -IV(STAGE)
         I = IV(XIRC)
!         GO TO (20, 30, 110, 110, 70), I
         SELECT CASE(I)
         CASE(1)
            GO TO 20
         CASE(2)
            GO TO 30
         CASE(3)
            GO TO 110
         CASE(4)
            GO TO 110
         CASE(5)
            GO TO 70
         END SELECT
!
 50   IF (IV(TOOBIG) .EQ. 0) GO TO 70
!
!  ***  HANDLE OVERSIZE STEP  ***
!
      IV(TOOBIG) = 0
      IF (IV(RADINC) .GT. 0) GO TO 80
         IV(STAGE) = -IV(STAGE)
         IV(XIRC) = IV(IRC)
!
 60      IV(TOOBIG) = 0
         V(RADFAC) = V(DECFAC)
         IV(RADINC) = IV(RADINC) - 1
         IV(IRC) = 5
         IV(RESTOR) = 1
         V(F) = V(FLSTGD)
         GO TO 999
!
 70   IF (V(F) .LT. V(FLSTGD)) GO TO 110
!
!     *** THE NEW STEP IS A LOSER.  RESTORE OLD MODEL.  ***
!
      IF (IV(MODEL) .EQ. IV(MLSTGD)) GO TO 80
         IV(MODEL) = IV(MLSTGD)
         IV(SWITCH) = 1
!
!     ***  RESTORE STEP, ETC. ONLY IF A PREVIOUS STEP DECREASED V(F).
!
 80   IF (V(FLSTGD) .GE. V(F0)) GO TO 110
         IF (IV(STAGE) .LT. IV(STGLIM)) THEN
            GOODX = .FALSE.
         ELSE IF (NFC .LT. IV(NFGCAL) + IV(STGLIM) + 2) THEN
            GOODX = .FALSE.
         ELSE IF (IV(SWITCH) .NE. 0) THEN
            GOODX = .FALSE.
            ENDIF
         IV(RESTOR) = 3
         V(F) = V(FLSTGD)
         V(PREDUC) = V(PLSTGD)
         V(GTSTEP) = V(GTSLST)
         IF (IV(SWITCH) .EQ. 0) RFAC1 = V(DSTNRM) / V(DSTSAV)
         V(DSTNRM) = V(DSTSAV)
         IF (GOODX) THEN
!
!     ***  ACCEPT PREVIOUS SLIGHTLY REDUCING STEP ***
!
            V(FDIF) = V(F0) - V(F)
            IV(IRC) = 4
            V(RADFAC) = RFAC1
            GO TO 999
            ENDIF
         NFC = IV(NFGCAL)
!
 110  V(FDIF) = V(F0) - V(F)
      IF (V(FDIF) .GT. V(TUNER2) * V(PREDUC)) GO TO 140
      IF (IV(RADINC) .GT. 0) GO TO 140
!
!        ***  NO (OR ONLY A TRIVIAL) FUNCTION DECREASE
!        ***  -- SO TRY NEW MODEL OR SMALLER RADIUS
!
         IF (V(F) .LT. V(F0)) GO TO 120
              IV(MLSTGD) = IV(MODEL)
              V(FLSTGD) = V(F)
              V(F) = V(F0)
              IV(RESTOR) = 1
              GO TO 130
 120     IV(NFGCAL) = NFC
 130     IV(IRC) = 1
         IF (IV(STAGE) .LT. IV(STGLIM)) GO TO 160
              IV(IRC) = 5
              IV(RADINC) = IV(RADINC) - 1
              GO TO 160
!
!  ***  NONTRIVIAL FUNCTION DECREASE ACHIEVED  ***
!
 140  IV(NFGCAL) = NFC
      RFAC1 = ONE
      V(DSTSAV) = V(DSTNRM)
      IF (V(FDIF) .GT. V(PREDUC)*V(TUNER1)) GO TO 190
!
!  ***  DECREASE WAS MUCH LESS THAN PREDICTED -- EITHER CHANGE MODELS
!  ***  OR ACCEPT STEP WITH DECREASED RADIUS.
!
      IF (IV(STAGE) .GE. IV(STGLIM)) GO TO 150
!        ***  CONSIDER SWITCHING MODELS  ***
         IV(IRC) = 2
         GO TO 160
!
!     ***  ACCEPT STEP WITH DECREASED RADIUS  ***
!
 150  IV(IRC) = 4
!
!  ***  SET V(RADFAC) TO FLETCHER*S DECREASE FACTOR  ***
!
 160  IV(XIRC) = IV(IRC)
      EMAX = V(GTSTEP) + V(FDIF)
      V(RADFAC) = HALF * RFAC1
      IF (EMAX .LT. V(GTSTEP)) V(RADFAC) = RFAC1 * MAX(V(RDFCMN), &
                                                 HALF * V(GTSTEP)/EMAX)
!
!  ***  DO FALSE CONVERGENCE TEST  ***
!
 170  IF (V(RELDX) .LE. V(XFTOL)) GO TO 180
         IV(IRC) = IV(XIRC)
         IF (V(F) .LT. V(F0)) GO TO 200
              GO TO 230
!
 180  IV(IRC) = 12
      GO TO 240
!
!  ***  HANDLE GOOD FUNCTION DECREASE  ***
!
 190  IF (V(FDIF) .LT. (-V(TUNER3) * V(GTSTEP))) GO TO 210
!
!     ***  INCREASING RADIUS LOOKS WORTHWHILE.  SEE IF WE JUST
!     ***  RECOMPUTED STEP WITH A DECREASED RADIUS OR RESTORED STEP
!     ***  AFTER RECOMPUTING IT WITH A LARGER RADIUS.
!
      IF (IV(RADINC) .LT. 0) GO TO 210
      IF (IV(RESTOR) .EQ. 1) GO TO 210
      IF (IV(RESTOR) .EQ. 3) GO TO 210
!
!        ***  WE DID NOT.  TRY A LONGER STEP UNLESS THIS WAS A NEWTON
!        ***  STEP.
!
         V(RADFAC) = V(RDFCMX)
         GTS = V(GTSTEP)
         IF (V(FDIF) .LT. (HALF/V(RADFAC) - ONE) * GTS) &
                  V(RADFAC) = MAX(V(INCFAC), HALF*GTS/(GTS + V(FDIF)))
         IV(IRC) = 4
         IF (V(STPPAR) .EQ. ZERO) GO TO 230
         IF (V(DST0) .GE. ZERO .AND. (V(DST0) .LT. TWO*V(DSTNRM) &
                   .OR. V(NREDUC) .LT. ONEP2*V(FDIF)))  GO TO 230
!             ***  STEP WAS NOT A NEWTON STEP.  RECOMPUTE IT WITH
!             ***  A LARGER RADIUS.
              IV(IRC) = 5
              IV(RADINC) = IV(RADINC) + 1
!
!  ***  SAVE VALUES CORRESPONDING TO GOOD STEP  ***
!
 200  V(FLSTGD) = V(F)
      IV(MLSTGD) = IV(MODEL)
      IF (IV(RESTOR) .EQ. 0) IV(RESTOR) = 2
      V(DSTSAV) = V(DSTNRM)
      IV(NFGCAL) = NFC
      V(PLSTGD) = V(PREDUC)
      V(GTSLST) = V(GTSTEP)
      GO TO 230
!
!  ***  ACCEPT STEP WITH RADIUS UNCHANGED  ***
!
 210  V(RADFAC) = ONE
      IV(IRC) = 3
      GO TO 230
!
!  ***  COME HERE FOR A RESTART AFTER CONVERGENCE  ***
!
 220  IV(IRC) = IV(XIRC)
      IF (V(DSTSAV) .GE. ZERO) GO TO 240
         IV(IRC) = 12
         GO TO 240
!
!  ***  PERFORM CONVERGENCE TESTS  ***
!
 230  IV(XIRC) = IV(IRC)
 240  IF (IV(RESTOR) .EQ. 1 .AND. V(FLSTGD) .LT. V(F0)) IV(RESTOR) = 3
      IF (ABS(V(F)) .LT. V(AFCTOL)) IV(IRC) = 10
      IF (HALF * V(FDIF) .GT. V(PREDUC)) GO TO 999
      EMAX = V(RFCTOL) * ABS(V(F0))
      EMAXS = V(SCTOL) * ABS(V(F0))
      IF (V(PREDUC) .LE. EMAXS .AND. (V(DSTNRM) .GT. V(LMAXS) .OR. &
           V(STPPAR) .EQ. ZERO)) IV(IRC) = 11
      IF (V(DST0) .LT. ZERO) GO TO 250
      I = 0
      IF ((V(NREDUC) .GT. ZERO .AND. V(NREDUC) .LE. EMAX) .OR. &
          (V(NREDUC) .EQ. ZERO .AND. V(PREDUC) .EQ. ZERO))  I = 2
      IF (V(STPPAR) .EQ. ZERO .AND. V(RELDX) .LE. V(XCTOL) &
                              .AND. GOODX)                  I = I + 1
      IF (I .GT. 0) IV(IRC) = I + 6
!
!  ***  CONSIDER RECOMPUTING STEP OF LENGTH V(LMAXS) FOR SINGULAR
!  ***  CONVERGENCE TEST.
!
 250  IF (IV(IRC) .GT. 5 .AND. IV(IRC) .NE. 12) GO TO 999
      IF (V(STPPAR) .EQ. ZERO) GO TO 999
      IF (V(DSTNRM) .GT. V(LMAXS)) GO TO 260
         IF (V(PREDUC) .GE. EMAXS) GO TO 999
              IF (V(DST0) .LE. ZERO) GO TO 270
                   IF (HALF * V(DST0) .LE. V(LMAXS)) GO TO 999
                        GO TO 270
 260  IF (HALF * V(DSTNRM) .LE. V(LMAXS)) GO TO 999
      XMAX = V(LMAXS) / V(DSTNRM)
      IF (XMAX * (TWO - XMAX) * V(PREDUC) .GE. EMAXS) GO TO 999
 270  IF (V(NREDUC) .LT. ZERO) GO TO 290
!
!  ***  RECOMPUTE V(PREDUC) FOR USE IN SINGULAR CONVERGENCE TEST  ***
!
      V(GTSLST) = V(GTSTEP)
      V(DSTSAV) = V(DSTNRM)
      IF (IV(IRC) .EQ. 12) V(DSTSAV) = -V(DSTSAV)
      V(PLSTGD) = V(PREDUC)
      I = IV(RESTOR)
      IV(RESTOR) = 2
      IF (I .EQ. 3) IV(RESTOR) = 0
      IV(IRC) = 6
      GO TO 999
!
!  ***  PERFORM SINGULAR CONVERGENCE TEST WITH RECOMPUTED V(PREDUC)  ***
!
 280  V(GTSTEP) = V(GTSLST)
      V(DSTNRM) = ABS(V(DSTSAV))
      IV(IRC) = IV(XIRC)
      IF (V(DSTSAV) .LE. ZERO) IV(IRC) = 12
      V(NREDUC) = -V(PREDUC)
      V(PREDUC) = V(PLSTGD)
      IV(RESTOR) = 3
 290  IF (-V(NREDUC) .LE. V(SCTOL) * ABS(V(F0))) IV(IRC) = 11
!
 999  RETURN
!
!  ***  LAST LINE OF DA7SST FOLLOWS  ***
      END
        SUBROUTINE DD7DGB(B, D, DIG, DST, G, IPIV, KA, L, LV, P, PC, &
                          NWTST, STEP, TD, TG, V, W, X0)
!
!  ***  COMPUTE DOUBLE-DOGLEG STEP, SUBJECT TO SIMPLE BOUNDS ON X  ***
!
      INTEGER LV, KA, P, PC
      INTEGER IPIV(P)
      DOUBLE PRECISION B(2,P), D(P), DIG(P), DST(P), G(P), L(1), &
                       NWTST(P), STEP(P), TD(P), TG(P), V(LV), W(P), &
                       X0(P)
!
!     DIMENSION L(P*(P+1)/2)
!
      DOUBLE PRECISION DD7TPR, DR7MDC, DV2NRM
      EXTERNAL DD7DOG, DD7TPR, I7SHFT, DL7ITV, DL7IVM, DL7TVM,DL7VML, &
               DQ7RSH, DR7MDC, DV2NRM,DV2AXY,DV7CPY, DV7IPR, DV7SCP, &
               DV7SHF, DV7VMP
!
!  ***  LOCAL VARIABLES  ***
!
      INTEGER I, J, K, P1, P1M1
      DOUBLE PRECISION DNWTST, GHINVG, GNORM, GNORM0, NRED, PRED, RAD, &
                       T, T1, T2, TI, X0I, XI
      DOUBLE PRECISION HALF, MEPS2, ONE, TWO, ZERO
!
!  ***  V SUBSCRIPTS  ***
!
      INTEGER DGNORM, DST0, DSTNRM, GRDFAC, GTHG, GTSTEP, NREDUC, &
              NWTFAC, PREDUC, RADIUS, STPPAR
!
!/6
!     DATA DGNORM/1/, DST0/3/, DSTNRM/2/, GRDFAC/45/, GTHG/44/,
!    1     GTSTEP/4/, NREDUC/6/, NWTFAC/46/, PREDUC/7/, RADIUS/8/,
!    2     STPPAR/5/
!/7
      PARAMETER (DGNORM=1, DST0=3, DSTNRM=2, GRDFAC=45, GTHG=44, &
                 GTSTEP=4, NREDUC=6, NWTFAC=46, PREDUC=7, RADIUS=8, &
                 STPPAR=5)
!/
!/6
!     DATA HALF/0.5D+0/, ONE/1.D+0/, TWO/2.D+0/, ZERO/0.D+0/
!/7
      PARAMETER (HALF=0.5D+0, ONE=1.D+0, TWO=2.D+0, ZERO=0.D+0)
      SAVE MEPS2
!/
      DATA MEPS2/0.D+0/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      IF (MEPS2 .LE. ZERO) MEPS2 = TWO * DR7MDC(3)
      GNORM0 = V(DGNORM)
      V(DSTNRM) = ZERO
      IF (KA .LT. 0) GO TO 10
         DNWTST = V(DST0)
         NRED = V(NREDUC)
 10   PRED = ZERO
      V(STPPAR) = ZERO
      RAD = V(RADIUS)
      IF (PC .GT. 0) GO TO 20
         DNWTST = ZERO
         CALL DV7SCP(P, STEP, ZERO)
         GO TO 140
!
 20   P1 = PC
      CALL DV7CPY(P, TD, D)
      CALL DV7IPR(P, IPIV, TD)
      CALL DV7SCP(PC, DST, ZERO)
      CALL DV7CPY(P, TG, G)
      CALL DV7IPR(P, IPIV, TG)
!
 30   CALL DL7IVM(P1, NWTST, L, TG)
      GHINVG = DD7TPR(P1, NWTST, NWTST)
      V(NREDUC) = HALF * GHINVG
      CALL DL7ITV(P1, NWTST, L, NWTST)
      CALL DV7VMP(P1, STEP, NWTST, TD, 1)
      V(DST0) = DV2NRM(PC, STEP)
      IF (KA .GE. 0) GO TO 40
         KA = 0
         DNWTST = V(DST0)
         NRED = V(NREDUC)
 40   V(RADIUS) = RAD - V(DSTNRM)
      IF (V(RADIUS) .LE. ZERO) GO TO 100
      CALL DV7VMP(P1, DIG, TG, TD, -1)
      GNORM = DV2NRM(P1, DIG)
      IF (GNORM .LE. ZERO) GO TO 100
      V(DGNORM) = GNORM
      CALL DV7VMP(P1, DIG, DIG, TD, -1)
      CALL DL7TVM(P1, W, L, DIG)
      V(GTHG) = DV2NRM(P1, W)
      KA = KA + 1
      CALL DD7DOG(DIG, LV, P1, NWTST, STEP, V)
!
!     ***  FIND T SUCH THAT X - T*STEP IS STILL FEASIBLE.
!
      T = ONE
      K = 0
      DO I = 1, P1
         J = IPIV(I)
         X0I = X0(J) + DST(I)/TD(I)
         XI = X0I + STEP(I)
         IF (XI .LT. B(1,J)) GO TO 50
         IF (XI .LE. B(2,J)) CYCLE 
              TI = (B(2,J) - X0I) / STEP(I)
              J = I
              GO TO 60
 50      TI = (B(1,J) - X0I) / STEP(I)
         J = -I
 60      IF (T .LE. TI) CYCLE 
              K = J
              T = TI
 70      CONTINUE
      END DO
!
!  ***  UPDATE DST, TG, AND PRED  ***
!
      CALL DV7VMP(P1, STEP, STEP, TD, 1)
      CALL DV2AXY(P1, DST, T, STEP, DST)
      V(DSTNRM) = DV2NRM(PC, DST)
      T1 = T * V(GRDFAC)
      T2 = T * V(NWTFAC)
      PRED = PRED - T1*GNORM * ((T2 + ONE)*GNORM) &
                       - T2 * (ONE + HALF*T2)*GHINVG &
                        - HALF * (V(GTHG)*T1)**2
      IF (K .EQ. 0) GO TO 100
      CALL DL7VML(P1, W, L, W)
      T2 = ONE - T2
      DO I = 1, P1
 80      TG(I) = T2*TG(I) - T1*W(I)
      END DO
!
!     ***  PERMUTE L, ETC. IF NECESSARY  ***
!
      P1M1 = P1 - 1
      J = ABS(K)
      IF (J .EQ. P1) GO TO 90
         CALL DQ7RSH(J, P1, .FALSE., TG, L, W)
         CALL I7SHFT(P1, J, IPIV)
         CALL DV7SHF(P1, J, TG)
         CALL DV7SHF(P1, J, TD)
         CALL DV7SHF(P1, J, DST)
 90   IF (K .LT. 0) IPIV(P1) = -IPIV(P1)
      P1 = P1M1
      IF (P1 .GT. 0) GO TO 30
!
!     ***  UNSCALE STEP, UPDATE X AND DIHDI  ***
!
 100  CALL DV7SCP(P, STEP, ZERO)
      DO I = 1, PC
         J = ABS(IPIV(I))
         STEP(J) = DST(I) / TD(I)
 110     CONTINUE
      END DO
!
!  ***  FUDGE STEP TO ENSURE THAT IT FORCES APPROPRIATE COMPONENTS
!  ***  TO THEIR BOUNDS  ***
!
      IF (P1 .GE. PC) GO TO 140
      CALL DV2AXY(P, TD, ONE, STEP, X0)
      K = P1 + 1
      DO I = K, PC
         J = IPIV(I)
         T = MEPS2
         IF (J .GT. 0) GO TO 120
            T = -T
            J = -J
            IPIV(I) = J
 120     T = T * MAX(ABS(TD(J)), ABS(X0(J)))
         STEP(J) = STEP(J) + T
 130     CONTINUE
      END DO
!
 140  V(DGNORM) = GNORM0
      V(NREDUC) = NRED
      V(PREDUC) = PRED
      V(RADIUS) = RAD
      V(DST0) = DNWTST
      V(GTSTEP) = DD7TPR(P, STEP, G)
!
 999  RETURN
!  ***  LAST LINE OF DD7DGB FOLLOWS  ***
      END
      SUBROUTINE DD7DOG(DIG, LV, N, NWTSTP, STEP, V)
!
!  ***  COMPUTE DOUBLE DOGLEG STEP  ***
!
!  ***  PARAMETER DECLARATIONS  ***
!
      INTEGER LV, N
      DOUBLE PRECISION DIG(N), NWTSTP(N), STEP(N), V(LV)
!
!  ***  PURPOSE  ***
!
!        THIS SUBROUTINE COMPUTES A CANDIDATE STEP (FOR USE IN AN UNCON-
!     STRAINED MINIMIZATION CODE) BY THE DOUBLE DOGLEG ALGORITHM OF
!     DENNIS AND MEI (REF. 1), WHICH IS A VARIATION ON POWELL*S DOGLEG
!     SCHEME (REF. 2, P. 95).
!
!--------------------------  PARAMETER USAGE  --------------------------
!
!    DIG (INPUT) DIAG(D)**-2 * G -- SEE ALGORITHM NOTES.
!      G (INPUT) THE CURRENT GRADIENT VECTOR.
!     LV (INPUT) LENGTH OF V.
!      N (INPUT) NUMBER OF COMPONENTS IN  DIG, G, NWTSTP,  AND  STEP.
! NWTSTP (INPUT) NEGATIVE NEWTON STEP -- SEE ALGORITHM NOTES.
!   STEP (OUTPUT) THE COMPUTED STEP.
!      V (I/O) VALUES ARRAY, THE FOLLOWING COMPONENTS OF WHICH ARE
!             USED HERE...
! V(BIAS)   (INPUT) BIAS FOR RELAXED NEWTON STEP, WHICH IS V(BIAS) OF
!             THE WAY FROM THE FULL NEWTON TO THE FULLY RELAXED NEWTON
!             STEP.  RECOMMENDED VALUE = 0.8 .
! V(DGNORM) (INPUT) 2-NORM OF DIAG(D)**-1 * G -- SEE ALGORITHM NOTES.
! V(DSTNRM) (OUTPUT) 2-NORM OF DIAG(D) * STEP, WHICH IS V(RADIUS)
!             UNLESS V(STPPAR) = 0 -- SEE ALGORITHM NOTES.
! V(DST0) (INPUT) 2-NORM OF DIAG(D) * NWTSTP -- SEE ALGORITHM NOTES.
! V(GRDFAC) (OUTPUT) THE COEFFICIENT OF  DIG  IN THE STEP RETURNED --
!             STEP(I) = V(GRDFAC)*DIG(I) + V(NWTFAC)*NWTSTP(I).
! V(GTHG)   (INPUT) SQUARE-ROOT OF (DIG**T) * (HESSIAN) * DIG -- SEE
!             ALGORITHM NOTES.
! V(GTSTEP) (OUTPUT) INNER PRODUCT BETWEEN G AND STEP.
! V(NREDUC) (OUTPUT) FUNCTION REDUCTION PREDICTED FOR THE FULL NEWTON
!             STEP.
! V(NWTFAC) (OUTPUT) THE COEFFICIENT OF  NWTSTP  IN THE STEP RETURNED --
!             SEE V(GRDFAC) ABOVE.
! V(PREDUC) (OUTPUT) FUNCTION REDUCTION PREDICTED FOR THE STEP RETURNED.
! V(RADIUS) (INPUT) THE TRUST REGION RADIUS.  D TIMES THE STEP RETURNED
!             HAS 2-NORM V(RADIUS) UNLESS V(STPPAR) = 0.
! V(STPPAR) (OUTPUT) CODE TELLING HOW STEP WAS COMPUTED... 0 MEANS A
!             FULL NEWTON STEP.  BETWEEN 0 AND 1 MEANS V(STPPAR) OF THE
!             WAY FROM THE NEWTON TO THE RELAXED NEWTON STEP.  BETWEEN
!             1 AND 2 MEANS A TRUE DOUBLE DOGLEG STEP, V(STPPAR) - 1 OF
!             THE WAY FROM THE RELAXED NEWTON TO THE CAUCHY STEP.
!             GREATER THAN 2 MEANS 1 / (V(STPPAR) - 1) TIMES THE CAUCHY
!             STEP.
!
!-------------------------------  NOTES  -------------------------------
!
!  ***  ALGORITHM NOTES  ***
!
!        LET  G  AND  H  BE THE CURRENT GRADIENT AND HESSIAN APPROXIMA-
!     TION RESPECTIVELY AND LET D BE THE CURRENT SCALE VECTOR.  THIS
!     ROUTINE ASSUMES DIG = DIAG(D)**-2 * G  AND  NWTSTP = H**-1 * G.
!     THE STEP COMPUTED IS THE SAME ONE WOULD GET BY REPLACING G AND H
!     BY  DIAG(D)**-1 * G  AND  DIAG(D)**-1 * H * DIAG(D)**-1,
!     COMPUTING STEP, AND TRANSLATING STEP BACK TO THE ORIGINAL
!     VARIABLES, I.E., PREMULTIPLYING IT BY DIAG(D)**-1.
!
!  ***  REFERENCES  ***
!
! 1.  DENNIS, J.E., AND MEI, H.H.W. (1979), TWO NEW UNCONSTRAINED OPTI-
!             MIZATION ALGORITHMS WHICH USE FUNCTION AND GRADIENT
!             VALUES, J. OPTIM. THEORY APPLIC. 28, PP. 453-482.
! 2. POWELL, M.J.D. (1970), A HYBRID METHOD FOR NON-LINEAR EQUATIONS,
!             IN NUMERICAL METHODS FOR NON-LINEAR EQUATIONS, EDITED BY
!             P. RABINOWITZ, GORDON AND BREACH, LONDON.
!
!  ***  GENERAL  ***
!
!     CODED BY DAVID M. GAY.
!     THIS SUBROUTINE WAS WRITTEN IN CONNECTION WITH RESEARCH SUPPORTED
!     BY THE NATIONAL SCIENCE FOUNDATION UNDER GRANTS MCS-7600324 AND
!     MCS-7906671.
!
!------------------------  EXTERNAL QUANTITIES  ------------------------
!
!  ***  INTRINSIC FUNCTIONS  ***
!/+
!      DOUBLE PRECISION DSQRT
!/
!--------------------------  LOCAL VARIABLES  --------------------------
!
      INTEGER I
      DOUBLE PRECISION CFACT, CNORM, CTRNWT, GHINVG, FEMNSQ, GNORM, &
                       NWTNRM, RELAX, RLAMBD, T, T1, T2
      DOUBLE PRECISION HALF, ONE, TWO, ZERO
!
!  ***  V SUBSCRIPTS  ***
!
      INTEGER BIAS, DGNORM, DSTNRM, DST0, GRDFAC, GTHG, GTSTEP, &
              NREDUC, NWTFAC, PREDUC, RADIUS, STPPAR
!
!  ***  DATA INITIALIZATIONS  ***
!
!/6
!     DATA HALF/0.5D+0/, ONE/1.D+0/, TWO/2.D+0/, ZERO/0.D+0/
!/7
      PARAMETER (HALF=0.5D+0, ONE=1.D+0, TWO=2.D+0, ZERO=0.D+0)
!/
!
!/6
!     DATA BIAS/43/, DGNORM/1/, DSTNRM/2/, DST0/3/, GRDFAC/45/,
!    1     GTHG/44/, GTSTEP/4/, NREDUC/6/, NWTFAC/46/, PREDUC/7/,
!    2     RADIUS/8/, STPPAR/5/
!/7
      PARAMETER (BIAS=43, DGNORM=1, DSTNRM=2, DST0=3, GRDFAC=45, &
                 GTHG=44, GTSTEP=4, NREDUC=6, NWTFAC=46, PREDUC=7, &
                 RADIUS=8, STPPAR=5)
!/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      NWTNRM = V(DST0)
      RLAMBD = ONE
      IF (NWTNRM .GT. ZERO) RLAMBD = V(RADIUS) / NWTNRM
      GNORM = V(DGNORM)
      GHINVG = TWO * V(NREDUC)
      V(GRDFAC) = ZERO
      V(NWTFAC) = ZERO
      IF (RLAMBD .LT. ONE) GO TO 30
!
!        ***  THE NEWTON STEP IS INSIDE THE TRUST REGION  ***
!
         V(STPPAR) = ZERO
         V(DSTNRM) = NWTNRM
         V(GTSTEP) = -GHINVG
         V(PREDUC) = V(NREDUC)
         V(NWTFAC) = -ONE
         DO I = 1, N
 20           STEP(I) = -NWTSTP(I)
         END DO
         GO TO 999
!
 30   V(DSTNRM) = V(RADIUS)
      CFACT = (GNORM / V(GTHG))**2
!     ***  CAUCHY STEP = -CFACT * G.
      CNORM = GNORM * CFACT
      RELAX = ONE - V(BIAS) * (ONE - GNORM*CNORM/GHINVG)
      IF (RLAMBD .LT. RELAX) GO TO 50
!
!        ***  STEP IS BETWEEN RELAXED NEWTON AND FULL NEWTON STEPS  ***
!
         V(STPPAR)  =  ONE  -  (RLAMBD - RELAX) / (ONE - RELAX)
         T = -RLAMBD
         V(GTSTEP) = T * GHINVG
         V(PREDUC) = RLAMBD * (ONE - HALF*RLAMBD) * GHINVG
         V(NWTFAC) = T
         DO I = 1, N
 40           STEP(I) = T * NWTSTP(I)
         END DO
         GO TO 999
!
 50   IF (CNORM .LT. V(RADIUS)) GO TO 70
!
!        ***  THE CAUCHY STEP LIES OUTSIDE THE TRUST REGION --
!        ***  STEP = SCALED CAUCHY STEP  ***
!
         T = -V(RADIUS) / GNORM
         V(GRDFAC) = T
         V(STPPAR) = ONE  +  CNORM / V(RADIUS)
         V(GTSTEP) = -V(RADIUS) * GNORM
      V(PREDUC) = V(RADIUS)*(GNORM - HALF*V(RADIUS)*(V(GTHG)/GNORM)**2)
         DO I = 1, N
 60           STEP(I) = T * DIG(I)
         END DO
         GO TO 999
!
!     ***  COMPUTE DOGLEG STEP BETWEEN CAUCHY AND RELAXED NEWTON  ***
!     ***  FEMUR = RELAXED NEWTON STEP MINUS CAUCHY STEP  ***
!
 70   CTRNWT = CFACT * RELAX * GHINVG / GNORM
!     *** CTRNWT = INNER PROD. OF CAUCHY AND RELAXED NEWTON STEPS,
!     *** SCALED BY GNORM**-1.
      T1 = CTRNWT - GNORM*CFACT**2
!     ***  T1 = INNER PROD. OF FEMUR AND CAUCHY STEP, SCALED BY
!     ***  GNORM**-1.
      T2 = V(RADIUS)*(V(RADIUS)/GNORM) - GNORM*CFACT**2
      T = RELAX * NWTNRM
      FEMNSQ = (T/GNORM)*T - CTRNWT - T1
!     ***  FEMNSQ = SQUARE OF 2-NORM OF FEMUR, SCALED BY GNORM**-1.
      T = T2 / (T1 + SQRT(T1**2 + FEMNSQ*T2))
!     ***  DOGLEG STEP  =  CAUCHY STEP  +  T * FEMUR.
      T1 = (T - ONE) * CFACT
      V(GRDFAC) = T1
      T2 = -T * RELAX
      V(NWTFAC) = T2
      V(STPPAR) = TWO - T
      V(GTSTEP) = T1*GNORM**2 + T2*GHINVG
      V(PREDUC) = -T1*GNORM * ((T2 + ONE)*GNORM) &
                       - T2 * (ONE + HALF*T2)*GHINVG &
                        - HALF * (V(GTHG)*T1)**2
      DO I = 1, N
 80      STEP(I) = T1*DIG(I) + T2*NWTSTP(I)
      END DO
!
 999  RETURN
!  ***  LAST LINE OF DD7DOG FOLLOWS  ***
      END
      DOUBLE PRECISION FUNCTION DD7TPR(P, X, Y)
!
!  ***  RETURN THE INNER PRODUCT OF THE P-VECTORS X AND Y.  ***
!
      INTEGER P
      DOUBLE PRECISION X(P), Y(P)
!
      INTEGER I
      DOUBLE PRECISION ONE, SQTETA, T, ZERO
      DOUBLE PRECISION DR7MDC
      EXTERNAL DR7MDC
!
!  ***  DR7MDC(2) RETURNS A MACHINE-DEPENDENT CONSTANT, SQTETA, WHICH
!  ***  IS SLIGHTLY LARGER THAN THE SMALLEST POSITIVE NUMBER THAT
!  ***  CAN BE SQUARED WITHOUT UNDERFLOWING.
!
!/6
!     DATA ONE/1.D+0/, SQTETA/0.D+0/, ZERO/0.D+0/
!/7
      PARAMETER (ONE=1.D+0, ZERO=0.D+0)
      DATA SQTETA/0.D+0/
!/
!
      DD7TPR = ZERO
      IF (P .LE. 0) GO TO 999
      IF (SQTETA .EQ. ZERO) SQTETA = DR7MDC(2)
      DO I = 1, P
         T = MAX(ABS(X(I)), ABS(Y(I)))
         IF (T .GT. ONE) GO TO 10
         IF (T .LT. SQTETA) CYCLE 
         T = (X(I)/SQTETA)*Y(I)
         IF (ABS(T) .LT. SQTETA) CYCLE
 10      DD7TPR = DD7TPR + X(I)*Y(I)
      END DO
 20   CONTINUE
!
 999  RETURN
!  ***  LAST LINE OF DD7TPR FOLLOWS  ***
      END
      SUBROUTINE DH2RFA(N, A, B, X, Y, Z)
!
!  ***  APPLY 2X2 HOUSEHOLDER REFLECTION DETERMINED BY X, Y, Z TO
!  ***  N-VECTORS A, B  ***
!
      INTEGER N
      DOUBLE PRECISION A(N), B(N), X, Y, Z
      INTEGER I
      DOUBLE PRECISION T
      DO I = 1, N
         T = A(I)*X + B(I)*Y
         A(I) = A(I) + T
         B(I) = B(I) + T*Z
 10      CONTINUE
      END DO
 999  RETURN
!  ***  LAST LINE OF DH2RFA FOLLOWS  ***
      END
      DOUBLE PRECISION FUNCTION DH2RFG(A, B, X, Y, Z)
!
!  ***  DETERMINE X, Y, Z SO  I + (1,Z)**T * (X,Y)  IS A 2X2
!  ***  HOUSEHOLDER REFLECTION SENDING (A,B)**T INTO (C,0)**T,
!  ***  WHERE  C = -SIGN(A)*SQRT(A**2 + B**2)  IS THE VALUE DH2RFG
!  ***  RETURNS.
!
      DOUBLE PRECISION A, B, X, Y, Z
!
      DOUBLE PRECISION A1, B1, C, T
!/+
!      DOUBLE PRECISION DSQRT
!/
      DOUBLE PRECISION ZERO
      DATA ZERO/0.D+0/
!
!  ***  BODY  ***
!
      IF (B .NE. ZERO) GO TO 10
         X = ZERO
         Y = ZERO
         Z = ZERO
         DH2RFG = A
         GO TO 999
 10   T = ABS(A) + ABS(B)
      A1 = A / T
      B1 = B / T
      C = SQRT(A1**2 + B1**2)
      IF (A1 .GT. ZERO) C = -C
      A1 = A1 - C
      Z = B1 / A1
      X = A1 / C
      Y = B1 / C
      DH2RFG = T * C
 999  RETURN
!  ***  LAST LINE OF DH2RFG FOLLOWS  ***
      END
      SUBROUTINE DITSUM(D, G, IV, LIV, LV, P, V, X)
!
!  ***  PRINT ITERATION SUMMARY FOR ***SOL (VERSION 2.3)  ***
!
!  ***  PARAMETER DECLARATIONS  ***
!
      INTEGER LIV, LV, P
      INTEGER IV(LIV)
      DOUBLE PRECISION D(P), G(P), V(LV), X(P)
!
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!  ***  LOCAL VARIABLES  ***
!
      INTEGER ALG, I, IV1, M, NF, NG, OL, PU
!/6S
!     REAL MODEL1(6), MODEL2(6)
!/7S
      CHARACTER(LEN=4) MODEL1(6), MODEL2(6)
!/
      DOUBLE PRECISION NRELDF, OLDF, PRELDF, RELDF, ZERO
!
!  ***  NO EXTERNAL FUNCTIONS OR SUBROUTINES  ***
!
!  ***  SUBSCRIPTS FOR IV AND V  ***
!
      INTEGER ALGSAV, DSTNRM, F, FDIF, F0, NEEDHD, NFCALL, NFCOV, NGCOV, &
              NGCALL, NITER, NREDUC, OUTLEV, PREDUC, PRNTIT, PRUNIT, &
              RELDX, SOLPRT, STATPR, STPPAR, SUSED, X0PRT
!
!  ***  IV SUBSCRIPT VALUES  ***
!
!/6
!     DATA ALGSAV/51/, NEEDHD/36/, NFCALL/6/, NFCOV/52/, NGCALL/30/,
!    1     NGCOV/53/, NITER/31/, OUTLEV/19/, PRNTIT/39/, PRUNIT/21/,
!    2     SOLPRT/22/, STATPR/23/, SUSED/64/, X0PRT/24/
!/7
      PARAMETER (ALGSAV=51, NEEDHD=36, NFCALL=6, NFCOV=52, NGCALL=30, &
                 NGCOV=53, NITER=31, OUTLEV=19, PRNTIT=39, PRUNIT=21, &
                 SOLPRT=22, STATPR=23, SUSED=64, X0PRT=24)
!/
!
!  ***  V SUBSCRIPT VALUES  ***
!
!/6
!     DATA DSTNRM/2/, F/10/, F0/13/, FDIF/11/, NREDUC/6/, PREDUC/7/,
!    1     RELDX/17/, STPPAR/5/
!/7
      PARAMETER (DSTNRM=2, F=10, F0=13, FDIF=11, NREDUC=6, PREDUC=7, &
                 RELDX=17, STPPAR=5)
!/
!
!/6
!     DATA ZERO/0.D+0/
!/7
      PARAMETER (ZERO=0.D+0)
!/
!/6S
!     DATA MODEL1(1)/4H    /, MODEL1(2)/4H    /, MODEL1(3)/4H    /,
!    1     MODEL1(4)/4H    /, MODEL1(5)/4H  G /, MODEL1(6)/4H  S /,
!    2     MODEL2(1)/4H G  /, MODEL2(2)/4H S  /, MODEL2(3)/4HG-S /,
!    3     MODEL2(4)/4HS-G /, MODEL2(5)/4H-S-G/, MODEL2(6)/4H-G-S/
!/7S
      DATA MODEL1/'    ','    ','    ','    ','  G ','  S '/, &
           MODEL2/' G  ',' S  ','G-S ','S-G ','-S-G','-G-S'/
!/
!
!-------------------------------  BODY  --------------------------------
!
      PU = IV(PRUNIT)
      IF (PU .EQ. 0) GO TO 999
      IV1 = IV(1)
      IF (IV1 .GT. 62) IV1 = IV1 - 51
      OL = IV(OUTLEV)
      ALG = MOD(IV(ALGSAV)-1,2) + 1
      IF (IV1 .LT. 2 .OR. IV1 .GT. 15) GO TO 370
      IF (IV1 .GE. 12) GO TO 120
      IF (IV1 .EQ. 2 .AND. IV(NITER) .EQ. 0) GO TO 390
      IF (OL .EQ. 0) GO TO 120
      IF (IV1 .GE. 10 .AND. IV(PRNTIT) .EQ. 0) GO TO 120
      IF (IV1 .GT. 2) GO TO 10
         IV(PRNTIT) = IV(PRNTIT) + 1
         IF (IV(PRNTIT) .LT. ABS(OL)) GO TO 999
 10   NF = IV(NFCALL) - ABS(IV(NFCOV))
      IV(PRNTIT) = 0
      RELDF = ZERO
      PRELDF = ZERO
      OLDF = MAX(ABS(V(F0)), ABS(V(F)))
      IF (OLDF .LE. ZERO) GO TO 20
         RELDF = V(FDIF) / OLDF
         PRELDF = V(PREDUC) / OLDF
 20   IF (OL .GT. 0) GO TO 60
!
!        ***  PRINT SHORT SUMMARY LINE  ***
!
         IF (IV(NEEDHD) .EQ. 1 .AND. ALG .EQ. 1) WRITE(PU,30)
 30   FORMAT(/"   IT   NF",6X,"F",7X,"RELDF",3X,"PRELDF",3X,"RELDX", &
             2X,"MODEL  STPPAR")
         IF (IV(NEEDHD) .EQ. 1 .AND. ALG .EQ. 2) WRITE(PU,40)
 40   FORMAT(/"    IT   NF",7X,"F",8X,"RELDF",4X,"PRELDF",4X,"RELDX", &
             3X,"STPPAR")
         IV(NEEDHD) = 0
         IF (ALG .EQ. 2) GO TO 50
         M = IV(SUSED)
         WRITE(PU,100) IV(NITER), NF, V(F), RELDF, PRELDF, V(RELDX), &
                       MODEL1(M), MODEL2(M), V(STPPAR)
         GO TO 120
!
 50      WRITE(PU,110) IV(NITER), NF, V(F), RELDF, PRELDF, V(RELDX), &
                       V(STPPAR)
         GO TO 120
!
!     ***  PRINT LONG SUMMARY LINE  ***
!
 60   IF (IV(NEEDHD) .EQ. 1 .AND. ALG .EQ. 1) WRITE(PU,70)
 70   FORMAT(/"    IT   NF",6X,"F",7X,"RELDF",3X,"PRELDF",3X,"RELDX", &
             2X,"MODEL  STPPAR",2X,"D*STEP",2X,"NPRELDF")
      IF (IV(NEEDHD) .EQ. 1 .AND. ALG .EQ. 2) WRITE(PU,80)
 80   FORMAT(/"    IT   NF",7X,"F",8X,"RELDF",4X,"PRELDF",4X,"RELDX", &
             3X,"STPPAR",3X,"D*STEP",3X,"NPRELDF")
      IV(NEEDHD) = 0
      NRELDF = ZERO
      IF (OLDF .GT. ZERO) NRELDF = V(NREDUC) / OLDF
      IF (ALG .EQ. 2) GO TO 90
      M = IV(SUSED)
      WRITE(PU,100) IV(NITER), NF, V(F), RELDF, PRELDF, V(RELDX), &
                   MODEL1(M), MODEL2(M), V(STPPAR), V(DSTNRM), NRELDF
      GO TO 120
!
 90   WRITE(PU,110) IV(NITER), NF, V(F), RELDF, PRELDF, &
                   V(RELDX), V(STPPAR), V(DSTNRM), NRELDF
 100  FORMAT(I6,I5,D10.3,2D9.2,D8.1,A3,A4,2D8.1,D9.2)
 110  FORMAT(I6,I5,D11.3,2D10.2,3D9.1,D10.2)
!
 120  IF (IV1 .LE. 2) GO TO 999
      I = IV(STATPR)
      IF (I .EQ. (-1)) GO TO 460
      IF (I + IV1 .LT. 0) GO TO 460
!      GO TO (999, 999, 130, 150, 170, 190, 210, 230, 250, 270, 290, 310, &
!             330, 350, 500),  IV1
      SELECT CASE(IV1)
      CASE(1)
         GO TO 999
      CASE(2)
         GO TO 999
      CASE(3)
         GO TO 130
      CASE(4)
         GO TO 150
      CASE(5)
         GO TO 170
      CASE(6)
         GO TO 190
      CASE(7)
         GO TO 210
      CASE(8)
         GO TO 230
      CASE(9)
         GO TO 250
      CASE(10)
         GO TO 270
      CASE(11)
         GO TO 290
      CASE(12)
         GO TO 310
      CASE(13)
         GO TO 330
      CASE(14)
         GO TO 350
      CASE(15)
         GO TO 500
      END SELECT
!
 130  WRITE(PU,140)
 140  FORMAT(/" ***** X-CONVERGENCE *****")
      GO TO 430
!
 150  WRITE(PU,160)
 160  FORMAT(/" ***** RELATIVE FUNCTION CONVERGENCE *****")
      GO TO 430
!
 170  WRITE(PU,180)
 180  FORMAT(/" ***** X- AND RELATIVE FUNCTION CONVERGENCE *****")
      GO TO 430
!
 190  WRITE(PU,200)
 200  FORMAT(/" ***** ABSOLUTE FUNCTION CONVERGENCE *****")
      GO TO 430
!
 210  WRITE(PU,220)
 220  FORMAT(/" ***** SINGULAR CONVERGENCE *****")
      GO TO 430
!
 230  WRITE(PU,240)
 240  FORMAT(/" ***** FALSE CONVERGENCE *****")
      GO TO 430
!
 250  WRITE(PU,260)
 260  FORMAT(/" ***** FUNCTION EVALUATION LIMIT *****")
      GO TO 430
!
 270  WRITE(PU,280)
 280  FORMAT(/" ***** ITERATION LIMIT *****")
      GO TO 430
!
 290  WRITE(PU,300)
 300  FORMAT(/" ***** STOPX *****")
      GO TO 430
!
 310  WRITE(PU,320)
 320  FORMAT(/" ***** INITIAL F(X) CANNOT BE COMPUTED *****")
!
      GO TO 390
!
 330  WRITE(PU,340)
 340  FORMAT(/" ***** BAD PARAMETERS TO ASSESS *****")
      GO TO 999
!
 350  WRITE(PU,360)
 360  FORMAT(/" ***** GRADIENT COULD NOT BE COMPUTED *****")
      IF (IV(NITER) .GT. 0) GO TO 460
      GO TO 390
!
 370  WRITE(PU,380) IV(1)
 380  FORMAT(/" ***** IV(1) =",I5," *****")
      GO TO 999
!
!  ***  INITIAL CALL ON DITSUM  ***
!
 390  IF (IV(X0PRT) .NE. 0) WRITE(PU,400) (I, X(I), D(I), I = 1, P)
 400  FORMAT(/"     I     INITIAL X(I)",8X,"D(I)"//(1X,I5,D17.6,D14.3))
!     *** THE FOLLOWING ARE TO AVOID UNDEFINED VARIABLES WHEN THE
!     *** FUNCTION EVALUATION LIMIT IS 1...
      V(DSTNRM) = ZERO
      V(FDIF) = ZERO
      V(NREDUC) = ZERO
      V(PREDUC) = ZERO
      V(RELDX) = ZERO
      IF (IV1 .GE. 12) GO TO 999
      IV(NEEDHD) = 0
      IV(PRNTIT) = 0
      IF (OL .EQ. 0) GO TO 999
      IF (OL .LT. 0 .AND. ALG .EQ. 1) WRITE(PU,30)
      IF (OL .LT. 0 .AND. ALG .EQ. 2) WRITE(PU,40)
      IF (OL .GT. 0 .AND. ALG .EQ. 1) WRITE(PU,70)
      IF (OL .GT. 0 .AND. ALG .EQ. 2) WRITE(PU,80)
      IF (ALG .EQ. 1) WRITE(PU,410) IV(NFCALL), V(F)
      IF (ALG .EQ. 2) WRITE(PU,420) IV(NFCALL), V(F)
 410  FORMAT(/"     0",I5,D10.3)
 420  FORMAT(/"     0",I5,D11.3)
      GO TO 999
!
!  ***  PRINT VARIOUS INFORMATION REQUESTED ON SOLUTION  ***
!
 430  IV(NEEDHD) = 1
      IF (IV(STATPR) .LE. 0) GO TO 460
         OLDF = MAX(ABS(V(F0)), ABS(V(F)))
         PRELDF = ZERO
         NRELDF = ZERO
         IF (OLDF .LE. ZERO) GO TO 440
              PRELDF = V(PREDUC) / OLDF
              NRELDF = V(NREDUC) / OLDF
 440     NF = IV(NFCALL) - IV(NFCOV)
         NG = IV(NGCALL) - IV(NGCOV)
         WRITE(PU,450) V(F), V(RELDX), NF, NG, PRELDF, NRELDF
 450  FORMAT(/" FUNCTION",D17.6,"   RELDX",D17.3/" FUNC. EVALS", &
         I8,9X,"GRAD. EVALS",I8/" PRELDF",D16.3,6X,"NPRELDF",D15.3)
!
 460  IF (IV(SOLPRT) .EQ. 0) GO TO 999
         IV(NEEDHD) = 1
         IF (IV(ALGSAV) .GT. 2) GO TO 999
         WRITE(PU,470)
 470  FORMAT(/"     I      FINAL X(I)",8X,"D(I)",10X,"G(I)"/)
         DO I = 1, P
 480          WRITE(PU,490) I, X(I), D(I), G(I)
         END DO
 490     FORMAT(1X,I5,D16.6,2D14.3)
      GO TO 999
!
 500  WRITE(PU,510)
 510  FORMAT(/" INCONSISTENT DIMENSIONS")
 999  RETURN
!  ***  LAST CARD OF DITSUM FOLLOWS  ***
      END
      SUBROUTINE DIVSET(ALG, IV, LIV, LV, V)
!
!  ***  SUPPLY ***SOL (VERSION 2.3) DEFAULT VALUES TO IV AND V  ***
!
!  ***  ALG = 1 MEANS REGRESSION CONSTANTS.
!  ***  ALG = 2 MEANS GENERAL UNCONSTRAINED OPTIMIZATION CONSTANTS.
!
      INTEGER LIV, LV
      INTEGER ALG, IV(LIV)
      DOUBLE PRECISION V(LV)
!
      INTEGER I7MDCN
      EXTERNAL I7MDCN,DV7DFL
! I7MDCN... RETURNS MACHINE-DEPENDENT INTEGER CONSTANTS.
! DV7DFL.... PROVIDES DEFAULT VALUES TO V.
!
      INTEGER ALG1, MIV, MV
      INTEGER MINIV(4), MINV(4)
!
!  ***  SUBSCRIPTS FOR IV  ***
!
      INTEGER ALGSAV, COVPRT, COVREQ, DRADPR, DTYPE, HC, IERR, INITH, &
              INITS, IPIVOT, IVNEED, LASTIV, LASTV, LMAT, MXFCAL, &
              MXITER, NFCOV, NGCOV, NVDFLT, NVSAVE, OUTLEV, PARPRT, &
              PARSAV, PERM, PRUNIT, QRTYP, RDREQ, RMAT, SOLPRT, STATPR, &
              VNEED, VSAVE, X0PRT
!
!  ***  IV SUBSCRIPT VALUES  ***
!
!/6
!     DATA ALGSAV/51/, COVPRT/14/, COVREQ/15/, DRADPR/101/, DTYPE/16/,
!    1     HC/71/, IERR/75/, INITH/25/, INITS/25/, IPIVOT/76/,
!    2     IVNEED/3/, LASTIV/44/, LASTV/45/, LMAT/42/, MXFCAL/17/,
!    3     MXITER/18/, NFCOV/52/, NGCOV/53/, NVDFLT/50/, NVSAVE/9/,
!    4     OUTLEV/19/, PARPRT/20/, PARSAV/49/, PERM/58/, PRUNIT/21/,
!    5     QRTYP/80/, RDREQ/57/, RMAT/78/, SOLPRT/22/, STATPR/23/,
!    6     VNEED/4/, VSAVE/60/, X0PRT/24/
!/7
      PARAMETER (ALGSAV=51, COVPRT=14, COVREQ=15, DRADPR=101, DTYPE=16, &
                 HC=71, IERR=75, INITH=25, INITS=25, IPIVOT=76, &
                 IVNEED=3, LASTIV=44, LASTV=45, LMAT=42, MXFCAL=17, &
                 MXITER=18, NFCOV=52, NGCOV=53, NVDFLT=50, NVSAVE=9, &
                 OUTLEV=19, PARPRT=20, PARSAV=49, PERM=58, PRUNIT=21, &
                 QRTYP=80, RDREQ=57, RMAT=78, SOLPRT=22, STATPR=23, &
                 VNEED=4, VSAVE=60, X0PRT=24)
!/
      DATA MINIV(1)/82/, MINIV(2)/59/, MINIV(3)/103/, MINIV(4)/103/, &
           MINV(1)/98/, MINV(2)/71/, MINV(3)/101/, MINV(4)/85/
!
!-------------------------------  BODY  --------------------------------
!
      IF (PRUNIT .LE. LIV) IV(PRUNIT) = I7MDCN(1)
      IF (ALGSAV .LE. LIV) IV(ALGSAV) = ALG
      IF (ALG .LT. 1 .OR. ALG .GT. 4) GO TO 40
      MIV = MINIV(ALG)
      IF (LIV .LT. MIV) GO TO 20
      MV = MINV(ALG)
      IF (LV .LT. MV) GO TO 30
      ALG1 = MOD(ALG-1,2) + 1
      CALL DV7DFL(ALG1, LV, V)
      IV(1) = 12
      IF (ALG .GT. 2) IV(DRADPR) = 1
      IV(IVNEED) = 0
      IV(LASTIV) = MIV
      IV(LASTV) = MV
      IV(LMAT) = MV + 1
      IV(MXFCAL) = 200
      IV(MXITER) = 150
      IV(OUTLEV) = 1
      IV(PARPRT) = 1
      IV(PERM) = MIV + 1
      IV(SOLPRT) = 1
      IV(STATPR) = 1
      IV(VNEED) = 0
      IV(X0PRT) = 1
!
      IF (ALG1 .GE. 2) GO TO 10
!
!  ***  REGRESSION  VALUES
!
      IV(COVPRT) = 3
      IV(COVREQ) = 1
      IV(DTYPE) = 1
      IV(HC) = 0
      IV(IERR) = 0
      IV(INITS) = 0
      IV(IPIVOT) = 0
      IV(NVDFLT) = 32
      IV(VSAVE) = 58
      IF (ALG .GT. 2) IV(VSAVE) = IV(VSAVE) + 3
      IV(PARSAV) = IV(VSAVE) + NVSAVE
      IV(QRTYP) = 1
      IV(RDREQ) = 3
      IV(RMAT) = 0
      GO TO 999
!
!  ***  GENERAL OPTIMIZATION VALUES
!
 10   IV(DTYPE) = 0
      IV(INITH) = 1
      IV(NFCOV) = 0
      IV(NGCOV) = 0
      IV(NVDFLT) = 25
      IV(PARSAV) = 47
      IF (ALG .GT. 2) IV(PARSAV) = 61
      GO TO 999
!
 20   IV(1) = 15
      GO TO 999
!
 30   IV(1) = 16
      GO TO 999
!
 40   IV(1) = 67
!
 999  RETURN
!  ***  LAST CARD OF DIVSET FOLLOWS  ***
      END
      SUBROUTINE DL7ITV(N, X, L, Y)
!
!  ***  SOLVE  (L**T)*X = Y,  WHERE  L  IS AN  N X N  LOWER TRIANGULAR
!  ***  MATRIX STORED COMPACTLY BY ROWS.  X AND Y MAY OCCUPY THE SAME
!  ***  STORAGE.  ***
!
      INTEGER N
      DOUBLE PRECISION X(N), L(1), Y(N)
      INTEGER I, II, IJ, IM1, I0, J, NP1
      DOUBLE PRECISION XI, ZERO
!/6
!     DATA ZERO/0.D+0/
!/7
      PARAMETER (ZERO=0.D+0)
!/
!
      DO I = 1, N
 10      X(I) = Y(I)
      END DO
      NP1 = N + 1
      I0 = N*(N+1)/2
      DO II = 1, N
         I = NP1 - II
         XI = X(I)/L(I0)
         X(I) = XI
         IF (I .LE. 1) GO TO 999
         I0 = I0 - I
         IF (XI .EQ. ZERO) CYCLE 
         IM1 = I - 1
         DO J = 1, IM1
              IJ = I0 + J
              X(J) = X(J) - XI*L(IJ)
 20           CONTINUE
         END DO
 30      CONTINUE
      END DO
 999  RETURN
!  ***  LAST CARD OF DL7ITV FOLLOWS  ***
      END
      SUBROUTINE DL7IVM(N, X, L, Y)
!
!  ***  SOLVE  L*X = Y, WHERE  L  IS AN  N X N  LOWER TRIANGULAR
!  ***  MATRIX STORED COMPACTLY BY ROWS.  X AND Y MAY OCCUPY THE SAME
!  ***  STORAGE.  ***
!
      INTEGER N
      DOUBLE PRECISION X(N), L(1), Y(N)
      DOUBLE PRECISION DD7TPR
      EXTERNAL DD7TPR
      INTEGER I, J, K
      DOUBLE PRECISION T, ZERO
!/6
!     DATA ZERO/0.D+0/
!/7
      PARAMETER (ZERO=0.D+0)
!/
!
      DO K = 1, N
         IF (Y(K) .NE. ZERO) GO TO 20
         X(K) = ZERO
 10      CONTINUE
      END DO
      GO TO 999
 20   J = K*(K+1)/2
      X(K) = Y(K) / L(J)
      IF (K .GE. N) GO TO 999
      K = K + 1
      DO I = K, N
         T = DD7TPR(I-1, L(J+1), X)
         J = J + I
         X(I) = (Y(I) - T)/L(J)
 30      CONTINUE
      END DO
 999  RETURN
!  ***  LAST CARD OF DL7IVM FOLLOWS  ***
      END
      SUBROUTINE DL7TVM(N, X, L, Y)
!
!  ***  COMPUTE  X = (L**T)*Y, WHERE  L  IS AN  N X N  LOWER
!  ***  TRIANGULAR MATRIX STORED COMPACTLY BY ROWS.  X AND Y MAY
!  ***  OCCUPY THE SAME STORAGE.  ***
!
      INTEGER N
      DOUBLE PRECISION X(N), L(1), Y(N)
!     DIMENSION L(N*(N+1)/2)
      INTEGER I, IJ, I0, J
      DOUBLE PRECISION YI, ZERO
!/6
!     DATA ZERO/0.D+0/
!/7
      PARAMETER (ZERO=0.D+0)
!/
!
      I0 = 0
      DO I = 1, N
         YI = Y(I)
         X(I) = ZERO
         DO J = 1, I
              IJ = I0 + J
              X(J) = X(J) + YI*L(IJ)
 10           CONTINUE
         END DO
         I0 = I0 + I
 20      CONTINUE
      END DO
 999  RETURN
!  ***  LAST CARD OF DL7TVM FOLLOWS  ***
      END
      SUBROUTINE DL7UPD(BETA, GAMMA, L, LAMBDA, LPLUS, N, W, Z)
!
!  ***  COMPUTE LPLUS = SECANT UPDATE OF L  ***
!
!  ***  PARAMETER DECLARATIONS  ***
!
      INTEGER N
      DOUBLE PRECISION BETA(N), GAMMA(N), L(1), LAMBDA(N), LPLUS(1), &
                       W(N), Z(N)
!     DIMENSION L(N*(N+1)/2), LPLUS(N*(N+1)/2)
!
!--------------------------  PARAMETER USAGE  --------------------------
!
!   BETA = SCRATCH VECTOR.
!  GAMMA = SCRATCH VECTOR.
!      L (INPUT) LOWER TRIANGULAR MATRIX, STORED ROWWISE.
! LAMBDA = SCRATCH VECTOR.
!  LPLUS (OUTPUT) LOWER TRIANGULAR MATRIX, STORED ROWWISE, WHICH MAY
!             OCCUPY THE SAME STORAGE AS  L.
!      N (INPUT) LENGTH OF VECTOR PARAMETERS AND ORDER OF MATRICES.
!      W (INPUT, DESTROYED ON OUTPUT) RIGHT SINGULAR VECTOR OF RANK 1
!             CORRECTION TO  L.
!      Z (INPUT, DESTROYED ON OUTPUT) LEFT SINGULAR VECTOR OF RANK 1
!             CORRECTION TO  L.
!
!-------------------------------  NOTES  -------------------------------
!
!  ***  APPLICATION AND USAGE RESTRICTIONS  ***
!
!        THIS ROUTINE UPDATES THE CHOLESKY FACTOR  L  OF A SYMMETRIC
!     POSITIVE DEFINITE MATRIX TO WHICH A SECANT UPDATE IS BEING
!     APPLIED -- IT COMPUTES A CHOLESKY FACTOR  LPLUS  OF
!     L * (I + Z*W**T) * (I + W*Z**T) * L**T.  IT IS ASSUMED THAT  W
!     AND  Z  HAVE BEEN CHOSEN SO THAT THE UPDATED MATRIX IS STRICTLY
!     POSITIVE DEFINITE.
!
!  ***  ALGORITHM NOTES  ***
!
!        THIS CODE USES RECURRENCE 3 OF REF. 1 (WITH D(J) = 1 FOR ALL J)
!     TO COMPUTE  LPLUS  OF THE FORM  L * (I + Z*W**T) * Q,  WHERE  Q
!     IS AN ORTHOGONAL MATRIX THAT MAKES THE RESULT LOWER TRIANGULAR.
!        LPLUS MAY HAVE SOME NEGATIVE DIAGONAL ELEMENTS.
!
!  ***  REFERENCES  ***
!
! 1.  GOLDFARB, D. (1976), FACTORIZED VARIABLE METRIC METHODS FOR UNCON-
!             STRAINED OPTIMIZATION, MATH. COMPUT. 30, PP. 796-811.
!
!  ***  GENERAL  ***
!
!     CODED BY DAVID M. GAY (FALL 1979).
!     THIS SUBROUTINE WAS WRITTEN IN CONNECTION WITH RESEARCH SUPPORTED
!     BY THE NATIONAL SCIENCE FOUNDATION UNDER GRANTS MCS-7600324 AND
!     MCS-7906671.
!
!------------------------  EXTERNAL QUANTITIES  ------------------------
!
!  ***  INTRINSIC FUNCTIONS  ***
!/+
!      DOUBLE PRECISION DSQRT
!/
!--------------------------  LOCAL VARIABLES  --------------------------
!
      INTEGER I, IJ, J, JJ, JP1, K, NM1, NP1
      DOUBLE PRECISION A, B, BJ, ETA, GJ, LJ, LIJ, LJJ, NU, S, THETA, &
                       WJ, ZJ
      DOUBLE PRECISION ONE, ZERO
!
!  ***  DATA INITIALIZATIONS  ***
!
!/6
!     DATA ONE/1.D+0/, ZERO/0.D+0/
!/7
      PARAMETER (ONE=1.D+0, ZERO=0.D+0)
!/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      NU = ONE
      ETA = ZERO
      IF (N .LE. 1) GO TO 30
      NM1 = N - 1
!
!  ***  TEMPORARILY STORE S(J) = SUM OVER K = J+1 TO N OF W(K)**2 IN
!  ***  LAMBDA(J).
!
      S = ZERO
      DO I = 1, NM1
         J = N - I
         S = S + W(J+1)**2
         LAMBDA(J) = S
 10      CONTINUE
      END DO
!
!  ***  COMPUTE LAMBDA, GAMMA, AND BETA BY GOLDFARB*S RECURRENCE 3.
!
      DO J = 1, NM1
         WJ = W(J)
         A = NU*Z(J) - ETA*WJ
         THETA = ONE + A*WJ
         S = A*LAMBDA(J)
         LJ = SQRT(THETA**2 + A*S)
         IF (THETA .GT. ZERO) LJ = -LJ
         LAMBDA(J) = LJ
         B = THETA*WJ + S
         GAMMA(J) = B * NU / LJ
         BETA(J) = (A - B*ETA) / LJ
         NU = -NU / LJ
         ETA = -(ETA + (A**2)/(THETA - LJ)) / LJ
 20      CONTINUE
      END DO
 30   LAMBDA(N) = ONE + (NU*Z(N) - ETA*W(N))*W(N)
!
!  ***  UPDATE L, GRADUALLY OVERWRITING  W  AND  Z  WITH  L*W  AND  L*Z.
!
      NP1 = N + 1
      JJ = N * (N + 1) / 2
      DO K = 1, N
         J = NP1 - K
         LJ = LAMBDA(J)
         LJJ = L(JJ)
         LPLUS(JJ) = LJ * LJJ
         WJ = W(J)
         W(J) = LJJ * WJ
         ZJ = Z(J)
         Z(J) = LJJ * ZJ
         IF (K .EQ. 1) GO TO 50
         BJ = BETA(J)
         GJ = GAMMA(J)
         IJ = JJ + J
         JP1 = J + 1
         DO I = JP1, N
              LIJ = L(IJ)
              LPLUS(IJ) = LJ*LIJ + BJ*W(I) + GJ*Z(I)
              W(I) = W(I) + LIJ*WJ
              Z(I) = Z(I) + LIJ*ZJ
              IJ = IJ + I
 40           CONTINUE
         END DO
 50      JJ = JJ - J
 60      CONTINUE
      END DO
!
 999  RETURN
!  ***  LAST CARD OF DL7UPD FOLLOWS  ***
      END
      SUBROUTINE DL7VML(N, X, L, Y)
!
!  ***  COMPUTE  X = L*Y, WHERE  L  IS AN  N X N  LOWER TRIANGULAR
!  ***  MATRIX STORED COMPACTLY BY ROWS.  X AND Y MAY OCCUPY THE SAME
!  ***  STORAGE.  ***
!
      INTEGER N
      DOUBLE PRECISION X(N), L(1), Y(N)
!     DIMENSION L(N*(N+1)/2)
      INTEGER I, II, IJ, I0, J, NP1
      DOUBLE PRECISION T, ZERO
!/6
!     DATA ZERO/0.D+0/
!/7
      PARAMETER (ZERO=0.D+0)
!/
!
      NP1 = N + 1
      I0 = N*(N+1)/2
      DO II = 1, N
         I = NP1 - II
         I0 = I0 - I
         T = ZERO
         DO J = 1, I
              IJ = I0 + J
              T = T + L(IJ)*Y(J)
 10           CONTINUE
         END DO
         X(I) = T
 20      CONTINUE
      END DO
 999  RETURN
!  ***  LAST CARD OF DL7VML FOLLOWS  ***
      END
      SUBROUTINE  DMNFB(P, D, X, B, CALCF, IV, LIV, LV, V, &
                        UIPARM, URPARM, UFPARM)
!
!  ***  MINIMIZE GENERAL UNCONSTRAINED OBJECTIVE FUNCTION USING
!  ***  FINITE-DIFFERENCE GRADIENTS AND SECANT HESSIAN APPROXIMATIONS.
!
      INTEGER P, LIV, LV
!/6S
!     INTEGER IV(LIV), UIPARM(1)
!     DOUBLE PRECISION B(2,P), D(P), X(P), V(LV), URPARM(1)
!/7S
      INTEGER IV(LIV), UIPARM(*)
      DOUBLE PRECISION B(2,P), D(P), X(P), V(LV), URPARM(*)
!/
!     DIMENSION V(59 + P), V(77 + P*(P+23)/2), UIPARM(*), URPARM(*)
      EXTERNAL CALCF, UFPARM
!
!  ***  PURPOSE  ***
!
!        THIS ROUTINE INTERACTS WITH SUBROUTINE  DRMNF  IN AN ATTEMPT
!     TO FIND AN P-VECTOR  X*  THAT MINIMIZES THE (UNCONSTRAINED)
!     OBJECTIVE FUNCTION COMPUTED BY  CALCF.  (OFTEN THE  X*  FOUND IS
!     A LOCAL MINIMIZER RATHER THAN A GLOBAL ONE.)
!
!  ***  PARAMETERS  ***
!
!        THE PARAMETERS FOR  DMNFB ARE THE SAME AS THOSE FOR  DMNGB
!     (WHICH SEE), EXCEPT THAT CALCG IS OMITTED.  INSTEAD OF CALLING
!     CALCG TO OBTAIN THE GRADIENT OF THE OBJECTIVE FUNCTION AT X,
!      DMNFB CALLS DS3GRD, WHICH COMPUTES AN APPROXIMATION TO THE
!     GRADIENT BY FINITE (FORWARD AND CENTRAL) DIFFERENCES USING THE
!     METHOD OF REF. 1.  THE FOLLOWING INPUT COMPONENT IS OF INTEREST
!     IN THIS REGARD (AND IS NOT DESCRIBED IN  DMNG OR  DMNGB).
!
! V(ETA0)..... V(42) IS AN ESTIMATED BOUND ON THE RELATIVE ERROR IN THE
!             OBJECTIVE FUNCTION VALUE COMPUTED BY CALCF...
!                  (TRUE VALUE) = (COMPUTED VALUE) * (1 + E),
!             WHERE ABS(E) .LE. V(ETA0).  DEFAULT = MACHEP * 10**3,
!             WHERE MACHEP IS THE UNIT ROUNDOFF.
!
!        THE OUTPUT VALUES IV(NFCALL) AND IV(NGCALL) HAVE DIFFERENT
!     MEANINGS FOR  DMNFB THAN FOR  DMNG...
!
! IV(NFCALL)... IV(6) IS THE NUMBER OF CALLS SO FAR MADE ON CALCF (I.E.,
!             FUNCTION EVALUATIONS) EXCLUDING THOSE MADE ONLY FOR
!             COMPUTING GRADIENTS.  THE INPUT VALUE IV(MXFCAL) IS A
!             LIMIT ON IV(NFCALL).
! IV(NGCALL)... IV(30) IS THE NUMBER OF FUNCTION EVALUATIONS MADE ONLY
!             FOR COMPUTING GRADIENTS.  THE TOTAL NUMBER OF FUNCTION
!             EVALUATIONS IS THUS  IV(NFCALL) + IV(NGCALL).
!
!  ***  REFERENCE  ***
!
! 1. STEWART, G.W. (1967), A MODIFICATION OF DAVIDON*S MINIMIZATION
!        METHOD TO ACCEPT DIFFERENCE APPROXIMATIONS OF DERIVATIVES,
!        J. ASSOC. COMPUT. MACH. 14, PP. 72-83.
!.
!  ***  GENERAL  ***
!
!     CODED BY DAVID M. GAY (WINTER 1980).  REVISED SEPT. 1982.
!     THIS SUBROUTINE WAS WRITTEN IN CONNECTION WITH RESEARCH
!     SUPPORTED IN PART BY THE NATIONAL SCIENCE FOUNDATION UNDER
!     GRANTS MCS-7600324, DCR75-10143, 76-14311DSS, MCS76-11989,
!     AND MCS-7906671.
!
!
!----------------------------  DECLARATIONS  ---------------------------
!
      EXTERNAL DRMNFB
!
! DRMNFB... OVERSEES COMPUTATION OF FINITE-DIFFERENCE GRADIENT AND
!         CALLS DRMNG TO CARRY OUT  DMNG ALGORITHM.
!
      INTEGER NF
      DOUBLE PRECISION FX
!
!  ***  SUBSCRIPTS FOR IV   ***
!
      INTEGER NFCALL, TOOBIG
!
!/6
!     DATA NFCALL/6/, TOOBIG/2/
!/7
      PARAMETER (NFCALL=6, TOOBIG=2)
!/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
 10   CALL DRMNFB(B, D, FX, IV, LIV, LV, P, V, X)
      IF (IV(1) .GT. 2) GO TO 999
!
!     ***  COMPUTE FUNCTION  ***
!
      NF = IV(NFCALL)
      CALL CALCF(P, X, NF, FX, UIPARM, URPARM, UFPARM)
      IF (NF .LE. 0) IV(TOOBIG) = 1
      GO TO 10
!
!
 999  RETURN
!  ***  LAST CARD OF  DMNFB FOLLOWS  ***
      END
      SUBROUTINE DPARCK(ALG, D, IV, LIV, LV, N, V)
!
!  ***  CHECK ***SOL (VERSION 2.3) PARAMETERS, PRINT CHANGED VALUES  ***
!
!  ***  ALG = 1 FOR REGRESSION, ALG = 2 FOR GENERAL UNCONSTRAINED OPT.
!
      INTEGER ALG, LIV, LV, N
      INTEGER IV(LIV)
      DOUBLE PRECISION D(N), V(LV)
!
      DOUBLE PRECISION DR7MDC
      EXTERNAL DIVSET, DR7MDC,DV7CPY,DV7DFL
! DIVSET  -- SUPPLIES DEFAULT VALUES TO BOTH IV AND V.
! DR7MDC -- RETURNS MACHINE-DEPENDENT CONSTANTS.
! DV7CPY  -- COPIES ONE VECTOR TO ANOTHER.
! DV7DFL  -- SUPPLIES DEFAULT PARAMETER VALUES TO V ALONE.
!
!  ***  LOCAL VARIABLES  ***
!
      INTEGER ALG1, I, II, IV1, J, K, L, M, MIV1, MIV2, NDFALT, PARSV1, &
              PU
      INTEGER IJMP, JLIM(4), MINIV(4), NDFLT(4)
!/6S
!     INTEGER VARNM(2), SH(2)
!     REAL CNGD(3), DFLT(3), VN(2,34), WHICH(3)
!/7S
      CHARACTER(LEN=1) VARNM(2), SH(2)
      CHARACTER(LEN=4) CNGD(3), DFLT(3), VN(2,34), WHICH(3)
!/
      DOUBLE PRECISION BIG, MACHEP, TINY, VK, VM(34), VX(34), ZERO
!
!  ***  IV AND V SUBSCRIPTS  ***
!
      INTEGER ALGSAV, DINIT, DTYPE, DTYPE0, EPSLON, INITS, IVNEED, &
              LASTIV, LASTV, LMAT, NEXTIV, NEXTV, NVDFLT, OLDN, &
              PARPRT, PARSAV, PERM, PRUNIT, VNEED
!
!
!/6
!     DATA ALGSAV/51/, DINIT/38/, DTYPE/16/, DTYPE0/54/, EPSLON/19/,
!    1     INITS/25/, IVNEED/3/, LASTIV/44/, LASTV/45/, LMAT/42/,
!    2     NEXTIV/46/, NEXTV/47/, NVDFLT/50/, OLDN/38/, PARPRT/20/,
!    3     PARSAV/49/, PERM/58/, PRUNIT/21/, VNEED/4/
!/7
      PARAMETER (ALGSAV=51, DINIT=38, DTYPE=16, DTYPE0=54, EPSLON=19, &
                 INITS=25, IVNEED=3, LASTIV=44, LASTV=45, LMAT=42, &
                 NEXTIV=46, NEXTV=47, NVDFLT=50, OLDN=38, PARPRT=20, &
                 PARSAV=49, PERM=58, PRUNIT=21, VNEED=4)
      SAVE BIG, MACHEP, TINY
!/
!
      DATA BIG/0.D+0/, MACHEP/-1.D+0/, TINY/1.D+0/, ZERO/0.D+0/
!/6S
!     DATA VN(1,1),VN(2,1)/4HEPSL,4HON../
!     DATA VN(1,2),VN(2,2)/4HPHMN,4HFC../
!     DATA VN(1,3),VN(2,3)/4HPHMX,4HFC../
!     DATA VN(1,4),VN(2,4)/4HDECF,4HAC../
!     DATA VN(1,5),VN(2,5)/4HINCF,4HAC../
!     DATA VN(1,6),VN(2,6)/4HRDFC,4HMN../
!     DATA VN(1,7),VN(2,7)/4HRDFC,4HMX../
!     DATA VN(1,8),VN(2,8)/4HTUNE,4HR1../
!     DATA VN(1,9),VN(2,9)/4HTUNE,4HR2../
!     DATA VN(1,10),VN(2,10)/4HTUNE,4HR3../
!     DATA VN(1,11),VN(2,11)/4HTUNE,4HR4../
!     DATA VN(1,12),VN(2,12)/4HTUNE,4HR5../
!     DATA VN(1,13),VN(2,13)/4HAFCT,4HOL../
!     DATA VN(1,14),VN(2,14)/4HRFCT,4HOL../
!     DATA VN(1,15),VN(2,15)/4HXCTO,4HL.../
!     DATA VN(1,16),VN(2,16)/4HXFTO,4HL.../
!     DATA VN(1,17),VN(2,17)/4HLMAX,4H0.../
!     DATA VN(1,18),VN(2,18)/4HLMAX,4HS.../
!     DATA VN(1,19),VN(2,19)/4HSCTO,4HL.../
!     DATA VN(1,20),VN(2,20)/4HDINI,4HT.../
!     DATA VN(1,21),VN(2,21)/4HDTIN,4HIT../
!     DATA VN(1,22),VN(2,22)/4HD0IN,4HIT../
!     DATA VN(1,23),VN(2,23)/4HDFAC,4H..../
!     DATA VN(1,24),VN(2,24)/4HDLTF,4HDC../
!     DATA VN(1,25),VN(2,25)/4HDLTF,4HDJ../
!     DATA VN(1,26),VN(2,26)/4HDELT,4HA0../
!     DATA VN(1,27),VN(2,27)/4HFUZZ,4H..../
!     DATA VN(1,28),VN(2,28)/4HRLIM,4HIT../
!     DATA VN(1,29),VN(2,29)/4HCOSM,4HIN../
!     DATA VN(1,30),VN(2,30)/4HHUBE,4HRC../
!     DATA VN(1,31),VN(2,31)/4HRSPT,4HOL../
!     DATA VN(1,32),VN(2,32)/4HSIGM,4HIN../
!     DATA VN(1,33),VN(2,33)/4HETA0,4H..../
!     DATA VN(1,34),VN(2,34)/4HBIAS,4H..../
!/7S
      DATA VN(1,1),VN(2,1)/'EPSL','ON..'/
      DATA VN(1,2),VN(2,2)/'PHMN','FC..'/
      DATA VN(1,3),VN(2,3)/'PHMX','FC..'/
      DATA VN(1,4),VN(2,4)/'DECF','AC..'/
      DATA VN(1,5),VN(2,5)/'INCF','AC..'/
      DATA VN(1,6),VN(2,6)/'RDFC','MN..'/
      DATA VN(1,7),VN(2,7)/'RDFC','MX..'/
      DATA VN(1,8),VN(2,8)/'TUNE','R1..'/
      DATA VN(1,9),VN(2,9)/'TUNE','R2..'/
      DATA VN(1,10),VN(2,10)/'TUNE','R3..'/
      DATA VN(1,11),VN(2,11)/'TUNE','R4..'/
      DATA VN(1,12),VN(2,12)/'TUNE','R5..'/
      DATA VN(1,13),VN(2,13)/'AFCT','OL..'/
      DATA VN(1,14),VN(2,14)/'RFCT','OL..'/
      DATA VN(1,15),VN(2,15)/'XCTO','L...'/
      DATA VN(1,16),VN(2,16)/'XFTO','L...'/
      DATA VN(1,17),VN(2,17)/'LMAX','0...'/
      DATA VN(1,18),VN(2,18)/'LMAX','S...'/
      DATA VN(1,19),VN(2,19)/'SCTO','L...'/
      DATA VN(1,20),VN(2,20)/'DINI','T...'/
      DATA VN(1,21),VN(2,21)/'DTIN','IT..'/
      DATA VN(1,22),VN(2,22)/'D0IN','IT..'/
      DATA VN(1,23),VN(2,23)/'DFAC','....'/
      DATA VN(1,24),VN(2,24)/'DLTF','DC..'/
      DATA VN(1,25),VN(2,25)/'DLTF','DJ..'/
      DATA VN(1,26),VN(2,26)/'DELT','A0..'/
      DATA VN(1,27),VN(2,27)/'FUZZ','....'/
      DATA VN(1,28),VN(2,28)/'RLIM','IT..'/
      DATA VN(1,29),VN(2,29)/'COSM','IN..'/
      DATA VN(1,30),VN(2,30)/'HUBE','RC..'/
      DATA VN(1,31),VN(2,31)/'RSPT','OL..'/
      DATA VN(1,32),VN(2,32)/'SIGM','IN..'/
      DATA VN(1,33),VN(2,33)/'ETA0','....'/
      DATA VN(1,34),VN(2,34)/'BIAS','....'/
!/
!
      DATA VM(1)/1.0D-3/, VM(2)/-0.99D+0/, VM(3)/1.0D-3/, VM(4)/1.0D-2/, &
           VM(5)/1.2D+0/, VM(6)/1.D-2/, VM(7)/1.2D+0/, VM(8)/0.D+0/, &
           VM(9)/0.D+0/, VM(10)/1.D-3/, VM(11)/-1.D+0/, VM(13)/0.D+0/, &
           VM(15)/0.D+0/, VM(16)/0.D+0/, VM(19)/0.D+0/, VM(20)/-10.D+0/, &
           VM(21)/0.D+0/, VM(22)/0.D+0/, VM(23)/0.D+0/, VM(27)/1.01D+0/, &
           VM(28)/1.D+10/, VM(30)/0.D+0/, VM(31)/0.D+0/, VM(32)/0.D+0/, &
           VM(34)/0.D+0/
      DATA VX(1)/0.9D+0/, VX(2)/-1.D-3/, VX(3)/1.D+1/, VX(4)/0.8D+0/, &
           VX(5)/1.D+2/, VX(6)/0.8D+0/, VX(7)/1.D+2/, VX(8)/0.5D+0/, &
           VX(9)/0.5D+0/, VX(10)/1.D+0/, VX(11)/1.D+0/, VX(14)/0.1D+0/, &
           VX(15)/1.D+0/, VX(16)/1.D+0/, VX(19)/1.D+0/, VX(23)/1.D+0/, &
           VX(24)/1.D+0/, VX(25)/1.D+0/, VX(26)/1.D+0/, VX(27)/1.D+10/, &
           VX(29)/1.D+0/, VX(31)/1.D+0/, VX(32)/1.D+0/, VX(33)/1.D+0/, &
           VX(34)/1.D+0/
!
!/6S
!     DATA VARNM(1)/1HP/, VARNM(2)/1HP/, SH(1)/1HS/, SH(2)/1HH/
!     DATA CNGD(1),CNGD(2),CNGD(3)/4H---C,4HHANG,4HED V/,
!    1     DFLT(1),DFLT(2),DFLT(3)/4HNOND,4HEFAU,4HLT V/
!/7S
      DATA VARNM(1)/'P'/, VARNM(2)/'P'/, SH(1)/'S'/, SH(2)/'H'/
      DATA CNGD(1),CNGD(2),CNGD(3)/'---C','HANG','ED V'/, &
           DFLT(1),DFLT(2),DFLT(3)/'NOND','EFAU','LT V'/
!/
      DATA IJMP/33/, JLIM(1)/0/, JLIM(2)/24/, JLIM(3)/0/, JLIM(4)/24/, &
           NDFLT(1)/32/, NDFLT(2)/25/, NDFLT(3)/32/, NDFLT(4)/25/
      DATA MINIV(1)/82/, MINIV(2)/59/, MINIV(3)/103/, MINIV(4)/103/
!
!...............................  BODY  ................................
!
      PU = 0
      IF (PRUNIT .LE. LIV) PU = IV(PRUNIT)
      IF (ALGSAV .GT. LIV) GO TO 20
      IF (ALG .EQ. IV(ALGSAV)) GO TO 20
         IF (PU .NE. 0) WRITE(PU,10) ALG, IV(ALGSAV)
 10      FORMAT(/" THE FIRST PARAMETER TO DIVSET SHOULD BE",I3, &
                " RATHER THAN",I3)
         IV(1) = 67
         GO TO 999
 20   IF (ALG .LT. 1 .OR. ALG .GT. 4) GO TO 340
      MIV1 = MINIV(ALG)
      IF (IV(1) .EQ. 15) GO TO 360
      ALG1 = MOD(ALG-1,2) + 1
      IF (IV(1) .EQ. 0) CALL DIVSET(ALG, IV, LIV, LV, V)
      IV1 = IV(1)
      IF (IV1 .NE. 13 .AND. IV1 .NE. 12) GO TO 30
      IF (PERM .LE. LIV) MIV1 = MAX0(MIV1, IV(PERM) - 1)
      IF (IVNEED .LE. LIV) MIV2 = MIV1 + MAX0(IV(IVNEED), 0)
      IF (LASTIV .LE. LIV) IV(LASTIV) = MIV2
      IF (LIV .LT. MIV1) GO TO 300
      IV(IVNEED) = 0
      IV(LASTV) = MAX0(IV(VNEED), 0) + IV(LMAT) - 1
      IV(VNEED) = 0
      IF (LIV .LT. MIV2) GO TO 300
      IF (LV .LT. IV(LASTV)) GO TO 320
 30   IF (IV1 .LT. 12 .OR. IV1 .GT. 14) GO TO 60
         IF (N .GE. 1) GO TO 50
              IV(1) = 81
              IF (PU .EQ. 0) GO TO 999
              WRITE(PU,40) VARNM(ALG1), N
 40           FORMAT(/" /// BAD",A1," =",I5)
              GO TO 999
 50      IF (IV1 .NE. 14) IV(NEXTIV) = IV(PERM)
         IF (IV1 .NE. 14) IV(NEXTV) = IV(LMAT)
         IF (IV1 .EQ. 13) GO TO 999
         K = IV(PARSAV) - EPSLON
         CALL DV7DFL(ALG1, LV-K, V(K+1))
         IV(DTYPE0) = 2 - ALG1
         IV(OLDN) = N
         WHICH(1) = DFLT(1)
         WHICH(2) = DFLT(2)
         WHICH(3) = DFLT(3)
         GO TO 110
 60   IF (N .EQ. IV(OLDN)) GO TO 80
         IV(1) = 17
         IF (PU .EQ. 0) GO TO 999
         WRITE(PU,70) VARNM(ALG1), IV(OLDN), N
 70      FORMAT(/" /// ",1A1," CHANGED FROM ",I5," TO ",I5)
         GO TO 999
!
 80   IF (IV1 .LE. 11 .AND. IV1 .GE. 1) GO TO 100
         IV(1) = 80
         IF (PU .NE. 0) WRITE(PU,90) IV1
 90      FORMAT(/" ///  IV(1) =",I5," SHOULD BE BETWEEN 0 AND 14.")
         GO TO 999
!
 100  WHICH(1) = CNGD(1)
      WHICH(2) = CNGD(2)
      WHICH(3) = CNGD(3)
!
 110  IF (IV1 .EQ. 14) IV1 = 12
      IF (BIG .GT. TINY) GO TO 120
         TINY = DR7MDC(1)
         MACHEP = DR7MDC(3)
         BIG = DR7MDC(6)
         VM(12) = MACHEP
         VX(12) = BIG
         VX(13) = BIG
         VM(14) = MACHEP
         VM(17) = TINY
         VX(17) = BIG
         VM(18) = TINY
         VX(18) = BIG
         VX(20) = BIG
         VX(21) = BIG
         VX(22) = BIG
         VM(24) = MACHEP
         VM(25) = MACHEP
         VM(26) = MACHEP
         VX(28) = DR7MDC(5)
         VM(29) = MACHEP
         VX(30) = BIG
         VM(33) = MACHEP
 120  M = 0
      I = 1
      J = JLIM(ALG1)
      K = EPSLON
      NDFALT = NDFLT(ALG1)
      DO L = 1, NDFALT
         VK = V(K)
         IF (VK .GE. VM(I) .AND. VK .LE. VX(I)) GO TO 140
              M = K
              IF (PU .NE. 0) WRITE(PU,130) VN(1,I), VN(2,I), K, VK, &
                                          VM(I), VX(I)
 130          FORMAT(/" ///  ",2A4,".. V(",I2,") =",D11.3," SHOULD", &
                     " BE BETWEEN",D11.3," AND",D11.3)
 140     K = K + 1
         I = I + 1
         IF (I .EQ. J) I = IJMP
 150     CONTINUE
      END DO
!
      IF (IV(NVDFLT) .EQ. NDFALT) GO TO 170
         IV(1) = 51
         IF (PU .EQ. 0) GO TO 999
         WRITE(PU,160) IV(NVDFLT), NDFALT
 160     FORMAT(/" IV(NVDFLT) =",I5," RATHER THAN ",I5)
         GO TO 999
 170  IF ((IV(DTYPE) .GT. 0 .OR. V(DINIT) .GT. ZERO) .AND. IV1 .EQ. 12) &
                        GO TO 200
      DO I = 1, N
         IF (D(I) .GT. ZERO) CYCLE 
              M = 18
              IF (PU .NE. 0) WRITE(PU,180) I, D(I)
 180     FORMAT(/" ///  D(",I3,") =",D11.3," SHOULD BE POSITIVE")
 190     CONTINUE
      END DO
 200  IF (M .EQ. 0) GO TO 210
         IV(1) = M
         GO TO 999
!
 210  IF (PU .EQ. 0 .OR. IV(PARPRT) .EQ. 0) GO TO 999
      IF (IV1 .NE. 12 .OR. IV(INITS) .EQ. ALG1-1) GO TO 230
         M = 1
         WRITE(PU,220) SH(ALG1), IV(INITS)
 220     FORMAT(/" NONDEFAULT VALUES...."/" INIT",A1,"..... IV(25) =", &
                I3)
 230  IF (IV(DTYPE) .EQ. IV(DTYPE0)) GO TO 250
         IF (M .EQ. 0) WRITE(PU,260) WHICH
         M = 1
         WRITE(PU,240) IV(DTYPE)
 240     FORMAT(" DTYPE..... IV(16) =",I3)
 250  I = 1
      J = JLIM(ALG1)
      K = EPSLON
      L = IV(PARSAV)
      NDFALT = NDFLT(ALG1)
      DO II = 1, NDFALT
         IF (V(K) .EQ. V(L)) GO TO 280
              IF (M .EQ. 0) WRITE(PU,260) WHICH
 260          FORMAT(/" ",3A4,"ALUES...."/)
              M = 1
              WRITE(PU,270) VN(1,I), VN(2,I), K, V(K)
 270          FORMAT(1X,2A4,".. V(",I2,") =",D15.7)
 280     K = K + 1
         L = L + 1
         I = I + 1
         IF (I .EQ. J) I = IJMP
 290     CONTINUE
      END DO
!
      IV(DTYPE0) = IV(DTYPE)
      PARSV1 = IV(PARSAV)
      CALL DV7CPY(IV(NVDFLT), V(PARSV1), V(EPSLON))
      GO TO 999
!
 300  IV(1) = 15
      IF (PU .EQ. 0) GO TO 999
      WRITE(PU,310) LIV, MIV2
 310  FORMAT(/" /// LIV =",I5," MUST BE AT LEAST",I5)
      IF (LIV .LT. MIV1) GO TO 999
      IF (LV .LT. IV(LASTV)) GO TO 320
      GO TO 999
!
 320  IV(1) = 16
      IF (PU .NE. 0) WRITE(PU,330) LV, IV(LASTV)
 330  FORMAT(/" /// LV =",I5," MUST BE AT LEAST",I5)
      GO TO 999
!
 340  IV(1) = 67
      IF (PU .NE. 0) WRITE(PU,350) ALG
 350  FORMAT(/" /// ALG =",I5," MUST BE 1 2, 3, OR 4")
      GO TO 999
 360  IF (PU .NE. 0) WRITE(PU,370) LIV, MIV1
 370  FORMAT(/" /// LIV =",I5," MUST BE AT LEAST",I5, &
             " TO COMPUTE TRUE MIN. LIV AND MIN. LV")
      IF (LASTIV .LE. LIV) IV(LASTIV) = MIV1
      IF (LASTV .LE. LIV) IV(LASTV) = 0
!
 999  RETURN
!  ***  LAST LINE OF DPARCK FOLLOWS  ***
      END
      SUBROUTINE DQ7RSH(K, P, HAVQTR, QTR, R, W)
!
!  ***  PERMUTE COLUMN K OF R TO COLUMN P, MODIFY QTR ACCORDINGLY  ***
!
      LOGICAL HAVQTR
      INTEGER K, P
      DOUBLE PRECISION QTR(P), R(1), W(P)
!     DIMSNSION R(P*(P+1)/2)
!
      DOUBLE PRECISION DH2RFG
      EXTERNAL DH2RFA, DH2RFG,DV7CPY
!
!  ***  LOCAL VARIABLES  ***
!
      INTEGER I, I1, J, JM1, JP1, J1, KM1, K1, PM1
      DOUBLE PRECISION A, B, T, WJ, X, Y, Z, ZERO
!
      DATA ZERO/0.0D+0/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      IF (K .GE. P) GO TO 999
      KM1 = K - 1
      K1 = K * KM1 / 2
      CALL DV7CPY(K, W, R(K1+1))
      WJ = W(K)
      PM1 = P - 1
      J1 = K1 + KM1
      DO J = K, PM1
         JM1 = J - 1
         JP1 = J + 1
         IF (JM1 .GT. 0) CALL DV7CPY(JM1, R(K1+1), R(J1+2))
         J1 = J1 + JP1
         K1 = K1 + J
         A = R(J1)
         B = R(J1+1)
         IF (B .NE. ZERO) GO TO 10
              R(K1) = A
              X = ZERO
              Z = ZERO
              GO TO 40
 10      R(K1) = DH2RFG(A, B, X, Y, Z)
         IF (J .EQ. PM1) GO TO 30
         I1 = J1
         DO 20 I = JP1, PM1
              I1 = I1 + I
              CALL DH2RFA(1, R(I1), R(I1+1), X, Y, Z)
 20           CONTINUE
 30      IF (HAVQTR) CALL DH2RFA(1, QTR(J), QTR(JP1), X, Y, Z)
 40      T = X * WJ
         W(J) = WJ + T
         WJ = T * Z
 50      CONTINUE
      END DO
      W(P) = WJ
      CALL DV7CPY(P, R(K1+1), W)
 999  RETURN
      END
      DOUBLE PRECISION FUNCTION DR7MDC(K)
!
!  ***  RETURN MACHINE DEPENDENT CONSTANTS USED BY NL2SOL  ***
!
      INTEGER K
!
!  ***  THE CONSTANT RETURNED DEPENDS ON K...
!
!  ***        K = 1... SMALLEST POS. ETA SUCH THAT -ETA EXISTS.
!  ***        K = 2... SQUARE ROOT OF ETA.
!  ***        K = 3... UNIT ROUNDOFF = SMALLEST POS. NO. MACHEP SUCH
!  ***                 THAT 1 + MACHEP .GT. 1 .AND. 1 - MACHEP .LT. 1.
!  ***        K = 4... SQUARE ROOT OF MACHEP.
!  ***        K = 5... SQUARE ROOT OF BIG (SEE K = 6).
!  ***        K = 6... LARGEST MACHINE NO. BIG SUCH THAT -BIG EXISTS.
!
      DOUBLE PRECISION BIG, ETA, MACHEP
!/+
!      DOUBLE PRECISION DSQRT
!/
!
      DOUBLE PRECISION D1MACH, ZERO
      EXTERNAL D1MACH
      DATA BIG/0.D+0/, ETA/0.D+0/, MACHEP/0.D+0/, ZERO/0.D+0/
      IF (BIG .GT. ZERO) GO TO 1
         BIG = D1MACH(2)
         ETA = D1MACH(1)
         MACHEP = D1MACH(4)
 1    CONTINUE
!
!-------------------------------  BODY  --------------------------------
!
!      GO TO (10, 20, 30, 40, 50, 60), K
      SELECT CASE(K)
      CASE(1)
         GO TO 10
      CASE(2)
         GO TO 20
      CASE(3)
         GO TO 30
      CASE(4)
         GO TO 40
      CASE(5)
         GO TO 50
      CASE(6)
         GO TO 60
      END SELECT
!
 10   DR7MDC = ETA
      GO TO 999
!
 20   DR7MDC = SQRT(256.D+0*ETA)/16.D+0
      GO TO 999
!
 30   DR7MDC = MACHEP
      GO TO 999
!
 40   DR7MDC = SQRT(MACHEP)
      GO TO 999
!
 50   DR7MDC = SQRT(BIG/256.D+0)*16.D+0
      GO TO 999
!
 60   DR7MDC = BIG
!
 999  RETURN
!  ***  LAST CARD OF DR7MDC FOLLOWS  ***
      END
      DOUBLE PRECISION FUNCTION DRLDST(P, D, X, X0)
!
!  ***  COMPUTE AND RETURN RELATIVE DIFFERENCE BETWEEN X AND X0  ***
!  ***  NL2SOL VERSION 2.2  ***
!
      INTEGER P
      DOUBLE PRECISION D(P), X(P), X0(P)
!
      INTEGER I
      DOUBLE PRECISION EMAX, T, XMAX, ZERO
!/6
!     DATA ZERO/0.D+0/
!/7
      PARAMETER (ZERO=0.D+0)
!/
!
!  ***  BODY  ***
!
      EMAX = ZERO
      XMAX = ZERO
      DO I = 1, P
         T = ABS(D(I) * (X(I) - X0(I)))
         IF (EMAX .LT. T) EMAX = T
         T = D(I) * (ABS(X(I)) + ABS(X0(I)))
         IF (XMAX .LT. T) XMAX = T
 10      CONTINUE
      END DO
      DRLDST = ZERO
      IF (XMAX .GT. ZERO) DRLDST = EMAX / XMAX
 999  RETURN
!  ***  LAST CARD OF DRLDST FOLLOWS  ***
      END
      SUBROUTINE DRMNFB(B, D, FX, IV, LIV, LV, P, V, X)
!
!  ***  ITERATION DRIVER FOR  DMNF...
!  ***  MINIMIZE GENERAL UNCONSTRAINED OBJECTIVE FUNCTION USING
!  ***  FINITE-DIFFERENCE GRADIENTS AND SECANT HESSIAN APPROXIMATIONS.
!
      INTEGER LIV, LV, P
      INTEGER IV(LIV)
      DOUBLE PRECISION B(2,P), D(P), FX, X(P), V(LV)
!     DIMENSION IV(59 + P), V(77 + P*(P+23)/2)
!
!  ***  PURPOSE  ***
!
!        THIS ROUTINE INTERACTS WITH SUBROUTINE  DRMNGB  IN AN ATTEMPT
!     TO FIND AN P-VECTOR  X*  THAT MINIMIZES THE (UNCONSTRAINED)
!     OBJECTIVE FUNCTION  FX = F(X)  COMPUTED BY THE CALLER.  (OFTEN
!     THE  X*  FOUND IS A LOCAL MINIMIZER RATHER THAN A GLOBAL ONE.)
!
!  ***  PARAMETERS  ***
!
!        THE PARAMETERS FOR DRMNFB ARE THE SAME AS THOSE FOR  DMNG
!     (WHICH SEE), EXCEPT THAT CALCF, CALCG, UIPARM, URPARM, AND UFPARM
!     ARE OMITTED, AND A PARAMETER  FX  FOR THE OBJECTIVE FUNCTION
!     VALUE AT X IS ADDED.  INSTEAD OF CALLING CALCG TO OBTAIN THE
!     GRADIENT OF THE OBJECTIVE FUNCTION AT X, DRMNFB CALLS DS3GRD,
!     WHICH COMPUTES AN APPROXIMATION TO THE GRADIENT BY FINITE
!     (FORWARD AND CENTRAL) DIFFERENCES USING THE METHOD OF REF. 1.
!     THE FOLLOWING INPUT COMPONENT IS OF INTEREST IN THIS REGARD
!     (AND IS NOT DESCRIBED IN  DMNG).
!
! V(ETA0)..... V(42) IS AN ESTIMATED BOUND ON THE RELATIVE ERROR IN THE
!             OBJECTIVE FUNCTION VALUE COMPUTED BY CALCF...
!                  (TRUE VALUE) = (COMPUTED VALUE) * (1 + E),
!             WHERE ABS(E) .LE. V(ETA0).  DEFAULT = MACHEP * 10**3,
!             WHERE MACHEP IS THE UNIT ROUNDOFF.
!
!        THE OUTPUT VALUES IV(NFCALL) AND IV(NGCALL) HAVE DIFFERENT
!     MEANINGS FOR  DMNF THAN FOR  DMNG...
!
! IV(NFCALL)... IV(6) IS THE NUMBER OF CALLS SO FAR MADE ON CALCF (I.E.,
!             FUNCTION EVALUATIONS) EXCLUDING THOSE MADE ONLY FOR
!             COMPUTING GRADIENTS.  THE INPUT VALUE IV(MXFCAL) IS A
!             LIMIT ON IV(NFCALL).
! IV(NGCALL)... IV(30) IS THE NUMBER OF FUNCTION EVALUATIONS MADE ONLY
!             FOR COMPUTING GRADIENTS.  THE TOTAL NUMBER OF FUNCTION
!             EVALUATIONS IS THUS  IV(NFCALL) + IV(NGCALL).
!
!  ***  REFERENCES  ***
!
! 1. STEWART, G.W. (1967), A MODIFICATION OF DAVIDON*S MINIMIZATION
!        METHOD TO ACCEPT DIFFERENCE APPROXIMATIONS OF DERIVATIVES,
!        J. ASSOC. COMPUT. MACH. 14, PP. 72-83.
!.
!  ***  GENERAL  ***
!
!     CODED BY DAVID M. GAY (AUGUST 1982).
!
!----------------------------  DECLARATIONS  ---------------------------
!
      DOUBLE PRECISION DD7TPR
      EXTERNAL DIVSET, DD7TPR, DS3GRD, DRMNGB, DV7SCP
!
! DIVSET.... SUPPLIES DEFAULT PARAMETER VALUES.
! DD7TPR... RETURNS INNER PRODUCT OF TWO VECTORS.
! DS3GRD... COMPUTES FINITE-DIFFERENCE GRADIENT APPROXIMATION.
! DRMNGB... REVERSE-COMMUNICATION ROUTINE THAT DOES  DMNGB ALGORITHM.
! DV7SCP... SETS ALL ELEMENTS OF A VECTOR TO A SCALAR.
!
      INTEGER ALPHA, ALPHA0, G1, I, IPI, IV1, J, K, W
      DOUBLE PRECISION ZERO
!
!  ***  SUBSCRIPTS FOR IV   ***
!
      INTEGER ETA0, F, G, LMAT, NEXTV, NGCALL, &
              NITER, PERM, SGIRC, TOOBIG, VNEED
!
!/6
!     DATA ETA0/42/, F/10/, G/28/, LMAT/42/, NEXTV/47/, NGCALL/30/,
!    1     NITER/31/, PERM/58/, SGIRC/57/, TOOBIG/2/, VNEED/4/
!/7
      PARAMETER (ETA0=42, F=10, G=28, LMAT=42, NEXTV=47, NGCALL=30, &
                 NITER=31, PERM=58, SGIRC=57, TOOBIG=2, VNEED=4)
!/
!/6
!     DATA ZERO/0.D+0/
!/7
      PARAMETER (ZERO=0.D+0)
!/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      IV1 = IV(1)
      IF (IV1 .EQ. 1) GO TO 10
      IF (IV1 .EQ. 2) GO TO 50
      IF (IV(1) .EQ. 0) CALL DIVSET(2, IV, LIV, LV, V)
      IV1 = IV(1)
      IF (IV1 .EQ. 12 .OR. IV1 .EQ. 13) IV(VNEED) = IV(VNEED) + 2*P + 6
      IF (IV1 .EQ. 14) GO TO 10
      IF (IV1 .GT. 2 .AND. IV1 .LT. 12) GO TO 10
      G1 = 1
      IF (IV1 .EQ. 12) IV(1) = 13
      GO TO 20
!
 10   G1 = IV(G)
!
 20   CALL DRMNGB(B, D, FX, V(G1), IV, LIV, LV, P, V, X)
!      IF (IV(1) - 2) 999, 30, 80
      IF (IV(1) - 2 .LT. 0) GO TO 999
      IF (IV(1) - 2 .EQ. 0) GO TO 30
      IF (IV(1) - 2 .GT. 0) GO TO 80
!
!  ***  COMPUTE GRADIENT  ***
!
 30   IF (IV(NITER) .EQ. 0) CALL DV7SCP(P, V(G1), ZERO)
      J = IV(LMAT)
      ALPHA0 = G1 - P - 1
      IPI = IV(PERM)
      DO I = 1, P
         K = ALPHA0 + IV(IPI)
         V(K) = DD7TPR(I, V(J), V(J))
         IPI = IPI + 1
         J = J + I
 40      CONTINUE
      END DO
!     ***  UNDO INCREMENT OF IV(NGCALL) DONE BY DRMNGB  ***
      IV(NGCALL) = IV(NGCALL) - 1
!     ***  STORE RETURN CODE FROM DS3GRD IN IV(SGIRC)  ***
      IV(SGIRC) = 0
!     ***  X MAY HAVE BEEN RESTORED, SO COPY BACK FX... ***
      FX = V(F)
      GO TO 60
!
!     ***  GRADIENT LOOP  ***
!
 50   IF (IV(TOOBIG) .NE. 0) GO TO 10
!
 60   G1 = IV(G)
      ALPHA = G1 - P
      W = ALPHA - 6
      CALL DS3GRD(V(ALPHA), B, D, V(ETA0), FX, V(G1), IV(SGIRC), P, &
                  V(W), X)
      I = IV(SGIRC)
      IF (I .EQ. 0) GO TO 10
      IF (I .LE. P) GO TO 70
         IV(TOOBIG) = 1
         GO TO 10
!
 70   IV(NGCALL) = IV(NGCALL) + 1
      GO TO 999
!
 80   IF (IV(1) .NE. 14) GO TO 999
!
!  ***  STORAGE ALLOCATION  ***
!
      IV(G) = IV(NEXTV) + P + 6
      IV(NEXTV) = IV(G) + P
      IF (IV1 .NE. 13) GO TO 10
!
 999  RETURN
!  ***  LAST CARD OF DRMNFB FOLLOWS  ***
      END
      SUBROUTINE DRMNGB(B, D, FX, G, IV, LIV, LV, N, V, X)
!
!  ***  CARRY OUT  DMNGB (SIMPLY BOUNDED MINIMIZATION) ITERATIONS,
!  ***  USING DOUBLE-DOGLEG/BFGS STEPS.
!
!  ***  PARAMETER DECLARATIONS  ***
!
      INTEGER LIV, LV, N
      INTEGER IV(LIV)
      DOUBLE PRECISION B(2,N), D(N), FX, G(N), V(LV), X(N)
!
!--------------------------  PARAMETER USAGE  --------------------------
!
! B.... VECTOR OF LOWER AND UPPER BOUNDS ON X.
! D.... SCALE VECTOR.
! FX... FUNCTION VALUE.
! G.... GRADIENT VECTOR.
! IV... INTEGER VALUE ARRAY.
! LIV.. LENGTH OF IV (AT LEAST 59) + N.
! LV... LENGTH OF V (AT LEAST 71 + N*(N+19)/2).
! N.... NUMBER OF VARIABLES (COMPONENTS IN X AND G).
! V.... FLOATING-POINT VALUE ARRAY.
! X.... VECTOR OF PARAMETERS TO BE OPTIMIZED.
!
!  ***  DISCUSSION  ***
!
!        PARAMETERS IV, N, V, AND X ARE THE SAME AS THE CORRESPONDING
!     ONES TO  DMNGB (WHICH SEE), EXCEPT THAT V CAN BE SHORTER (SINCE
!     THE PART OF V THAT  DMNGB USES FOR STORING G IS NOT NEEDED).
!     MOREOVER, COMPARED WITH  DMNGB, IV(1) MAY HAVE THE TWO ADDITIONAL
!     OUTPUT VALUES 1 AND 2, WHICH ARE EXPLAINED BELOW, AS IS THE USE
!     OF IV(TOOBIG) AND IV(NFGCAL).  THE VALUE IV(G), WHICH IS AN
!     OUTPUT VALUE FROM  DMNGB (AND SMSNOB), IS NOT REFERENCED BY
!     DRMNGB OR THE SUBROUTINES IT CALLS.
!        FX AND G NEED NOT HAVE BEEN INITIALIZED WHEN DRMNGB IS CALLED
!     WITH IV(1) = 12, 13, OR 14.
!
! IV(1) = 1 MEANS THE CALLER SHOULD SET FX TO F(X), THE FUNCTION VALUE
!             AT X, AND CALL DRMNGB AGAIN, HAVING CHANGED NONE OF THE
!             OTHER PARAMETERS.  AN EXCEPTION OCCURS IF F(X) CANNOT BE
!             (E.G. IF OVERFLOW WOULD OCCUR), WHICH MAY HAPPEN BECAUSE
!             OF AN OVERSIZED STEP.  IN THIS CASE THE CALLER SHOULD SET
!             IV(TOOBIG) = IV(2) TO 1, WHICH WILL CAUSE DRMNGB TO IG-
!             NORE FX AND TRY A SMALLER STEP.  THE PARAMETER NF THAT
!              DMNGB PASSES TO CALCF (FOR POSSIBLE USE BY CALCG) IS A
!             COPY OF IV(NFCALL) = IV(6).
! IV(1) = 2 MEANS THE CALLER SHOULD SET G TO G(X), THE GRADIENT VECTOR
!             OF F AT X, AND CALL DRMNGB AGAIN, HAVING CHANGED NONE OF
!             THE OTHER PARAMETERS EXCEPT POSSIBLY THE SCALE VECTOR D
!             WHEN IV(DTYPE) = 0.  THE PARAMETER NF THAT  DMNGB PASSES
!             TO CALCG IS IV(NFGCAL) = IV(7).  IF G(X) CANNOT BE
!             EVALUATED, THEN THE CALLER MAY SET IV(NFGCAL) TO 0, IN
!             WHICH CASE DRMNGB WILL RETURN WITH IV(1) = 65.
!.
!  ***  GENERAL  ***
!
!     CODED BY DAVID M. GAY (DECEMBER 1979).  REVISED SEPT. 1982.
!     THIS SUBROUTINE WAS WRITTEN IN CONNECTION WITH RESEARCH SUPPORTED
!     IN PART BY THE NATIONAL SCIENCE FOUNDATION UNDER GRANTS
!     MCS-7600324 AND MCS-7906671.
!
!        (SEE  DMNG FOR REFERENCES.)
!
!+++++++++++++++++++++++++++  DECLARATIONS  ++++++++++++++++++++++++++++
!
!  ***  LOCAL VARIABLES  ***
!
      INTEGER DG1, DSTEP1, DUMMY, G01, I, I1, IPI, IPN, J, K, L, LSTGST, &
              N1, NP1, NWTST1, RSTRST, STEP1, TEMP0, TEMP1, TD1, TG1, &
              W1, X01, Z
      DOUBLE PRECISION GI, T, XI
!
!     ***  CONSTANTS  ***
!
      DOUBLE PRECISION NEGONE, ONE, ONEP2, ZERO
!
!  ***  NO INTRINSIC FUNCTIONS  ***
!
!  ***  EXTERNAL FUNCTIONS AND SUBROUTINES  ***
!
      LOGICAL STOPX
      DOUBLE PRECISION DD7TPR, DRLDST, DV2NRM
      EXTERNAL DA7SST, DD7DGB,DIVSET, DD7TPR, I7SHFT,DITSUM, DL7TVM, &
               DL7UPD,DL7VML,DPARCK, DQ7RSH, DRLDST, STOPX, DV2NRM, &
              DV2AXY,DV7CPY, DV7IPR, DV7SCP, DV7VMP, DW7ZBF
!
! DA7SST.... ASSESSES CANDIDATE STEP.
! DD7DGB... COMPUTES SIMPLY BOUNDED DOUBLE-DOGLEG (CANDIDATE) STEP.
! DIVSET.... SUPPLIES DEFAULT IV AND V INPUT COMPONENTS.
! DD7TPR... RETURNS INNER PRODUCT OF TWO VECTORS.
! I7SHFT... CYCLICALLLY SHIFTS AN ARRAY OF INTEGERS.
! DITSUM.... PRINTS ITERATION SUMMARY AND INFO ON INITIAL AND FINAL X.
! DL7TVM... MULTIPLIES TRANSPOSE OF LOWER TRIANGLE TIMES VECTOR.
! LUPDT.... UPDATES CHOLESKY FACTOR OF HESSIAN APPROXIMATION.
! DL7VML.... MULTIPLIES LOWER TRIANGLE TIMES VECTOR.
! DPARCK.... CHECKS VALIDITY OF INPUT IV AND V VALUES.
! DQ7RSH... CYCLICALLY SHIFTS CHOLESKY FACTOR.
! DRLDST... COMPUTES V(RELDX) = RELATIVE STEP SIZE.
! STOPX.... RETURNS .TRUE. IF THE BREAK KEY HAS BEEN PRESSED.
! DV2NRM... RETURNS THE 2-NORM OF A VECTOR.
! DV2AXY.... COMPUTES SCALAR TIMES ONE VECTOR PLUS ANOTHER.
! DV7CPY.... COPIES ONE VECTOR TO ANOTHER.
! DV7IPR... CYCLICALLY SHIFTS A FLOATING-POINT ARRAY.
! DV7SCP... SETS ALL ELEMENTS OF A VECTOR TO A SCALAR.
! DV7VMP... MULTIPLIES VECTOR BY VECTOR RAISED TO POWER (COMPONENTWISE).
! DW7ZBF... COMPUTES W AND Z FOR DL7UPD CORRESPONDING TO BFGS UPDATE.
!
!  ***  SUBSCRIPTS FOR IV AND V  ***
!
      INTEGER CNVCOD, DG, DGNORM, DINIT, DSTNRM, F, F0, FDIF, &
              GTSTEP, INCFAC, INITH, IRC, IVNEED, KAGQT, LMAT, &
              LMAX0, LMAXS, MODE, MODEL, MXFCAL, MXITER, NC, NEXTIV, &
              NEXTV, NFCALL, NFGCAL, NGCALL, NITER, NWTSTP, PERM, &
              PREDUC, RADFAC, RADINC, RADIUS, RAD0, RELDX, RESTOR, STEP, &
              STGLIM, STLSTG, TOOBIG, TUNER4, TUNER5, VNEED, XIRC, X0
!
!  ***  IV SUBSCRIPT VALUES  ***
!
!  ***  (NOTE THAT NC IS STORED IN IV(G0)) ***
!
!/6
!     DATA CNVCOD/55/, DG/37/, INITH/25/, IRC/29/, IVNEED/3/, KAGQT/33/,
!    1     MODE/35/, MODEL/5/, MXFCAL/17/, MXITER/18/, NC/48/,
!    2     NEXTIV/46/, NEXTV/47/, NFCALL/6/, NFGCAL/7/, NGCALL/30/,
!    3     NITER/31/, NWTSTP/34/, PERM/58/, RADINC/8/, RESTOR/9/,
!    4     STEP/40/, STGLIM/11/, STLSTG/41/, TOOBIG/2/, XIRC/13/, X0/43/
!/7
      PARAMETER (CNVCOD=55, DG=37, INITH=25, IRC=29, IVNEED=3, KAGQT=33, &
                 MODE=35, MODEL=5, MXFCAL=17, MXITER=18, NC=48, &
                 NEXTIV=46, NEXTV=47, NFCALL=6, NFGCAL=7, NGCALL=30, &
                 NITER=31, NWTSTP=34, PERM=58, RADINC=8, RESTOR=9, &
                 STEP=40, STGLIM=11, STLSTG=41, TOOBIG=2, XIRC=13, &
                 X0=43)
!/
!
!  ***  V SUBSCRIPT VALUES  ***
!
!/6
!     DATA DGNORM/1/, DINIT/38/, DSTNRM/2/, F/10/, F0/13/, FDIF/11/,
!    1     GTSTEP/4/, INCFAC/23/, LMAT/42/, LMAX0/35/, LMAXS/36/,
!    2     PREDUC/7/, RADFAC/16/, RADIUS/8/, RAD0/9/, RELDX/17/,
!    3     TUNER4/29/, TUNER5/30/, VNEED/4/
!/7
      PARAMETER (DGNORM=1, DINIT=38, DSTNRM=2, F=10, F0=13, FDIF=11, &
                 GTSTEP=4, INCFAC=23, LMAT=42, LMAX0=35, LMAXS=36, &
                 PREDUC=7, RADFAC=16, RADIUS=8, RAD0=9, RELDX=17, &
                 TUNER4=29, TUNER5=30, VNEED=4)
!/
!
!/6
!     DATA NEGONE/-1.D+0/, ONE/1.D+0/, ONEP2/1.2D+0/, ZERO/0.D+0/
!/7
      PARAMETER (NEGONE=-1.D+0, ONE=1.D+0, ONEP2=1.2D+0, ZERO=0.D+0)
!/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      I = IV(1)
      IF (I .EQ. 1) GO TO 70
      IF (I .EQ. 2) GO TO 80
!
!  ***  CHECK VALIDITY OF IV AND V INPUT VALUES  ***
!
      IF (IV(1) .EQ. 0) CALL DIVSET(2, IV, LIV, LV, V)
      IF (IV(1) .LT. 12) GO TO 10
      IF (IV(1) .GT. 13) GO TO 10
         IV(VNEED) = IV(VNEED) + N*(N+19)/2
         IV(IVNEED) = IV(IVNEED) + N
 10   CALL DPARCK(2, D, IV, LIV, LV, N, V)
      I = IV(1) - 2
      IF (I .GT. 12) GO TO 999
!      GO TO (250, 250, 250, 250, 250, 250, 190, 150, 190, 20, 20, 30), I
      SELECT CASE(I)
      CASE(1)
         GO TO 250
      CASE(2)
         GO TO 250
      CASE(3)
         GO TO 250
      CASE(4)
         GO TO 250
      CASE(5)
         GO TO 250
      CASE(6)
         GO TO 250
      CASE(7)
         GO TO 190
      CASE(8)
         GO TO 150
      CASE(9)
         GO TO 190
      CASE(10)
         GO TO 20
      CASE(11)
         GO TO 20
      CASE(12)
         GO TO 30
      END SELECT
!
!  ***  STORAGE ALLOCATION  ***
!
 20   L = IV(LMAT)
      IV(X0) = L + N*(N+1)/2
      IV(STEP) = IV(X0) + 2*N
      IV(STLSTG) = IV(STEP) + 2*N
      IV(NWTSTP) = IV(STLSTG) + N
      IV(DG) = IV(NWTSTP) + 2*N
      IV(NEXTV) = IV(DG) + 2*N
      IV(NEXTIV) = IV(PERM) + N
      IF (IV(1) .NE. 13) GO TO 30
         IV(1) = 14
         GO TO 999
!
!  ***  INITIALIZATION  ***
!
 30   IV(NITER) = 0
      IV(NFCALL) = 1
      IV(NGCALL) = 1
      IV(NFGCAL) = 1
      IV(MODE) = -1
      IV(MODEL) = 1
      IV(STGLIM) = 1
      IV(TOOBIG) = 0
      IV(CNVCOD) = 0
      IV(RADINC) = 0
      IV(NC) = N
      V(RAD0) = ZERO
!
!  ***  CHECK CONSISTENCY OF B AND INITIALIZE IP ARRAY  ***
!
      IPI = IV(PERM)
      DO I = 1, N
         IV(IPI) = I
         IPI = IPI + 1
         IF (B(1,I) .GT. B(2,I)) GO TO 410
 40      CONTINUE
      END DO
!
      IF (V(DINIT) .GE. ZERO) CALL DV7SCP(N, D, V(DINIT))
      IF (IV(INITH) .NE. 1) GO TO 60
!
!     ***  SET THE INITIAL HESSIAN APPROXIMATION TO DIAG(D)**-2  ***
!
         L = IV(LMAT)
         CALL DV7SCP(N*(N+1)/2, V(L), ZERO)
         K = L - 1
         DO I = 1, N
              K = K + I
              T = D(I)
              IF (T .LE. ZERO) T = ONE
              V(K) = T
 50           CONTINUE
         END DO
!
!  ***  GET INITIAL FUNCTION VALUE  ***
!
 60   IV(1) = 1
      GO TO 440
!
 70   V(F) = FX
      IF (IV(MODE) .GE. 0) GO TO 250
      V(F0) = FX
      IV(1) = 2
      IF (IV(TOOBIG) .EQ. 0) GO TO 999
         IV(1) = 63
         GO TO 430
!
!  ***  MAKE SURE GRADIENT COULD BE COMPUTED  ***
!
 80   IF (IV(TOOBIG) .EQ. 0) GO TO 90
         IV(1) = 65
         GO TO 430
!
!  ***  CHOOSE INITIAL PERMUTATION  ***
!
 90   IPI = IV(PERM)
      IPN = IPI + N
      N1 = N
      NP1 = N + 1
      L = IV(LMAT)
      W1 = IV(NWTSTP) + N
      K = N - IV(NC)
      DO I = 1, N
         IPN = IPN - 1
         J = IV(IPN)
         IF (B(1,J) .GE. B(2,J)) GO TO 100
         XI = X(J)
         GI = G(J)
         IF (XI .LE. B(1,J) .AND. GI .GT. ZERO) GO TO 100
         IF (XI .GE. B(2,J) .AND. GI .LT. ZERO) GO TO 100
!           *** DISALLOW CONVERGENCE IF X(J) HAS JUST BEEN FREED ***
            IF (I .LE. K) IV(CNVCOD) = 0
            CYCLE
 100     I1 = NP1 - I
         IF (I1 .GE. N1) GO TO 110
            CALL I7SHFT(N1, I1, IV(IPI))
            CALL DQ7RSH(I1, N1, .FALSE., G, V(L), V(W1))
 110        N1 = N1 - 1
 120     CONTINUE
      END DO
!
      IV(NC) = N1
      V(DGNORM) = ZERO
      IF (N1 .LE. 0) GO TO 130
         DG1 = IV(DG)
         CALL DV7VMP(N, V(DG1), G, D, -1)
         CALL DV7IPR(N, IV(IPI), V(DG1))
         V(DGNORM) = DV2NRM(N1, V(DG1))
 130  IF (IV(CNVCOD) .NE. 0) GO TO 420
      IF (IV(MODE) .EQ. 0) GO TO 370
!
!  ***  ALLOW FIRST STEP TO HAVE SCALED 2-NORM AT MOST V(LMAX0)  ***
!
      V(RADIUS) = V(LMAX0)
!
      IV(MODE) = 0
!
!
!-----------------------------  MAIN LOOP  -----------------------------
!
!
!  ***  PRINT ITERATION SUMMARY, CHECK ITERATION LIMIT  ***
!
 140  CALL DITSUM(D, G, IV, LIV, LV, N, V, X)
 150  K = IV(NITER)
      IF (K .LT. IV(MXITER)) GO TO 160
         IV(1) = 10
         GO TO 430
!
!  ***  UPDATE RADIUS  ***
!
 160  IV(NITER) = K + 1
      IF (K .EQ. 0) GO TO 170
      T = V(RADFAC) * V(DSTNRM)
      IF (V(RADFAC) .LT. ONE .OR. T .GT. V(RADIUS)) V(RADIUS) = T
!
!  ***  INITIALIZE FOR START OF NEXT ITERATION  ***
!
 170  X01 = IV(X0)
      V(F0) = V(F)
      IV(IRC) = 4
      IV(KAGQT) = -1
!
!     ***  COPY X TO X0  ***
!
      CALL DV7CPY(N, V(X01), X)
!
!  ***  CHECK STOPX AND FUNCTION EVALUATION LIMIT  ***
!
 180  IF (.NOT. STOPX(DUMMY)) GO TO 200
         IV(1) = 11
         GO TO 210
!
!     ***  COME HERE WHEN RESTARTING AFTER FUNC. EVAL. LIMIT OR STOPX.
!
 190  IF (V(F) .GE. V(F0)) GO TO 200
         V(RADFAC) = ONE
         K = IV(NITER)
         GO TO 160
!
 200  IF (IV(NFCALL) .LT. IV(MXFCAL)) GO TO 220
         IV(1) = 9
 210     IF (V(F) .GE. V(F0)) GO TO 430
!
!        ***  IN CASE OF STOPX OR FUNCTION EVALUATION LIMIT WITH
!        ***  IMPROVED V(F), EVALUATE THE GRADIENT AT X.
!
              IV(CNVCOD) = IV(1)
              GO TO 360
!
!. . . . . . . . . . . . .  COMPUTE CANDIDATE STEP  . . . . . . . . . .
!
 220  STEP1 = IV(STEP)
      DG1 = IV(DG)
      NWTST1 = IV(NWTSTP)
      W1 = NWTST1 + N
      DSTEP1 = STEP1 + N
      IPI = IV(PERM)
      L = IV(LMAT)
      TG1 = DG1 + N
      X01 = IV(X0)
      TD1 = X01 + N
      CALL DD7DGB(B, D, V(DG1), V(DSTEP1), G, IV(IPI), IV(KAGQT), &
                  V(L), LV, N, IV(NC), V(NWTST1), V(STEP1), V(TD1), &
                  V(TG1), V, V(W1), V(X01))
      IF (IV(IRC) .NE. 6) GO TO 230
         IF (IV(RESTOR) .NE. 2) GO TO 250
         RSTRST = 2
         GO TO 260
!
!  ***  CHECK WHETHER EVALUATING F(X0 + STEP) LOOKS WORTHWHILE  ***
!
 230  IV(TOOBIG) = 0
      IF (V(DSTNRM) .LE. ZERO) GO TO 250
      IF (IV(IRC) .NE. 5) GO TO 240
      IF (V(RADFAC) .LE. ONE) GO TO 240
      IF (V(PREDUC) .GT. ONEP2 * V(FDIF)) GO TO 240
         IF (IV(RESTOR) .NE. 2) GO TO 250
         RSTRST = 0
         GO TO 260
!
!  ***  COMPUTE F(X0 + STEP)  ***
!
 240  CALL DV2AXY(N, X, ONE, V(STEP1), V(X01))
      IV(NFCALL) = IV(NFCALL) + 1
      IV(1) = 1
      GO TO 440
!
!. . . . . . . . . . . . .  ASSESS CANDIDATE STEP  . . . . . . . . . . .
!
 250  RSTRST = 3
 260  X01 = IV(X0)
      V(RELDX) = DRLDST(N, D, X, V(X01))
      CALL DA7SST(IV, LIV, LV, V)
      STEP1 = IV(STEP)
      LSTGST = IV(STLSTG)
      I = IV(RESTOR) + 1
!      GO TO (300, 270, 280, 290), I
      SELECT CASE(I)
      CASE(1)
         GO TO 300
      CASE(2)
         GO TO 270
      CASE(3)
         GO TO 280
      CASE(4)
         GO TO 290
      END SELECT
 270  CALL DV7CPY(N, X, V(X01))
      GO TO 300
 280   CALL DV7CPY(N, V(LSTGST), X)
       GO TO 300
 290     CALL DV7CPY(N, X, V(LSTGST))
         CALL DV2AXY(N, V(STEP1), NEGONE, V(X01), X)
         V(RELDX) = DRLDST(N, D, X, V(X01))
         IV(RESTOR) = RSTRST
!
 300  K = IV(IRC)
!      GO TO (310,340,340,340,310,320,330,330,330,330,330,330,400,370), K
      SELECT CASE(K)
      CASE(1)
         GO TO 310
      CASE(2)
         GO TO 340
      CASE(3)
         GO TO 340
      CASE(4)
         GO TO 340
      CASE(5)
         GO TO 310
      CASE(6)
         GO TO 320
      CASE(7)
         GO TO 330
      CASE(8)
         GO TO 330
      CASE(9)
         GO TO 330
      CASE(10)
         GO TO 330
      CASE(11)
         GO TO 330
      CASE(12)
         GO TO 330
      CASE(13)
         GO TO 400
      CASE(14)
         GO TO 370
      END SELECT
!
!     ***  RECOMPUTE STEP WITH CHANGED RADIUS  ***
!
 310     V(RADIUS) = V(RADFAC) * V(DSTNRM)
         GO TO 180
!
!  ***  COMPUTE STEP OF LENGTH V(LMAXS) FOR SINGULAR CONVERGENCE TEST.
!
 320  V(RADIUS) = V(LMAXS)
      GO TO 220
!
!  ***  CONVERGENCE OR FALSE CONVERGENCE  ***
!
 330  IV(CNVCOD) = K - 4
      IF (V(F) .GE. V(F0)) GO TO 420
         IF (IV(XIRC) .EQ. 14) GO TO 420
              IV(XIRC) = 14
!
!. . . . . . . . . . . .  PROCESS ACCEPTABLE STEP  . . . . . . . . . . .
!
 340  X01 = IV(X0)
      STEP1 = IV(STEP)
      CALL DV2AXY(N, V(STEP1), NEGONE, V(X01), X)
      IF (IV(IRC) .NE. 3) GO TO 360
!
!     ***  SET  TEMP1 = HESSIAN * STEP  FOR USE IN GRADIENT TESTS  ***
!
!     ***  USE X0 AS TEMPORARY...
!
         IPI = IV(PERM)
         CALL DV7CPY(N, V(X01), V(STEP1))
         CALL DV7IPR(N, IV(IPI), V(X01))
         L = IV(LMAT)
         CALL DL7TVM(N, V(X01), V(L), V(X01))
         CALL DL7VML(N, V(X01), V(L), V(X01))
!
!        *** UNPERMUTE X0 INTO TEMP1 ***
!
         TEMP1 = IV(STLSTG)
         TEMP0 = TEMP1 - 1
         DO I = 1, N
            J = IV(IPI)
            IPI = IPI + 1
            K = TEMP0 + J
            V(K) = V(X01)
            X01 = X01 + 1
 350        CONTINUE
         END DO
!
!  ***  SAVE OLD GRADIENT, COMPUTE NEW ONE  ***
!
 360  G01 = IV(NWTSTP) + N
      CALL DV7CPY(N, V(G01), G)
      IV(NGCALL) = IV(NGCALL) + 1
      IV(TOOBIG) = 0
      IV(1) = 2
      GO TO 999
!
!  ***  INITIALIZATIONS -- G0 = G - G0, ETC.  ***
!
 370  G01 = IV(NWTSTP) + N
      CALL DV2AXY(N, V(G01), NEGONE, V(G01), G)
      STEP1 = IV(STEP)
      TEMP1 = IV(STLSTG)
      IF (IV(IRC) .NE. 3) GO TO 390
!
!  ***  SET V(RADFAC) BY GRADIENT TESTS  ***
!
!     ***  SET  TEMP1 = DIAG(D)**-1 * (HESSIAN*STEP + (G(X0)-G(X)))  ***
!
         CALL DV2AXY(N, V(TEMP1), NEGONE, V(G01), V(TEMP1))
         CALL DV7VMP(N, V(TEMP1), V(TEMP1), D, -1)
!
!        ***  DO GRADIENT TESTS  ***
!
         IF (DV2NRM(N, V(TEMP1)) .LE. V(DGNORM) * V(TUNER4)) &
                        GO TO 380
              IF (DD7TPR(N, G, V(STEP1)) &
                        .GE. V(GTSTEP) * V(TUNER5))  GO TO 390
 380               V(RADFAC) = V(INCFAC)
!
!  ***  UPDATE H, LOOP  ***
!
 390  W1 = IV(NWTSTP)
      Z = IV(X0)
      L = IV(LMAT)
      IPI = IV(PERM)
      CALL DV7IPR(N, IV(IPI), V(STEP1))
      CALL DV7IPR(N, IV(IPI), V(G01))
      CALL DW7ZBF(V(L), N, V(STEP1), V(W1), V(G01), V(Z))
!
!     ** USE THE N-VECTORS STARTING AT V(STEP1) AND V(G01) FOR SCRATCH..
      CALL DL7UPD(V(TEMP1), V(STEP1), V(L), V(G01), V(L), N, V(W1), &
                  V(Z))
      IV(1) = 2
      GO TO 140
!
!. . . . . . . . . . . . . .  MISC. DETAILS  . . . . . . . . . . . . . .
!
!  ***  BAD PARAMETERS TO ASSESS  ***
!
 400  IV(1) = 64
      GO TO 430
!
!  ***  INCONSISTENT B  ***
!
 410  IV(1) = 82
      GO TO 430
!
!  ***  PRINT SUMMARY OF FINAL ITERATION AND OTHER REQUESTED ITEMS  ***
!
 420  IV(1) = IV(CNVCOD)
      IV(CNVCOD) = 0
 430  CALL DITSUM(D, G, IV, LIV, LV, N, V, X)
      GO TO 999
!
!  ***  PROJECT X INTO FEASIBLE REGION (PRIOR TO COMPUTING F OR G)  ***
!
 440  DO I = 1, N
         IF (X(I) .LT. B(1,I)) X(I) = B(1,I)
         IF (X(I) .GT. B(2,I)) X(I) = B(2,I)
 450     CONTINUE
      END DO
!
 999  RETURN
!
!  ***  LAST CARD OF DRMNGB FOLLOWS  ***
      END
      SUBROUTINE DS3GRD(ALPHA, B, D, ETA0, FX, G, IRC, P, W, X)
!
!  ***  COMPUTE FINITE DIFFERENCE GRADIENT BY STWEART*S SCHEME  ***
!
!     ***  PARAMETERS  ***
!
      INTEGER IRC, P
      DOUBLE PRECISION ALPHA(P), B(2,P), D(P), ETA0, FX, G(P), W(6), &
                       X(P)
!
!.......................................................................
!
!     ***  PURPOSE  ***
!
!        THIS SUBROUTINE USES AN EMBELLISHED FORM OF THE FINITE-DIFFER-
!     ENCE SCHEME PROPOSED BY STEWART (REF. 1) TO APPROXIMATE THE
!     GRADIENT OF THE FUNCTION F(X), WHOSE VALUES ARE SUPPLIED BY
!     REVERSE COMMUNICATION.
!
!     ***  PARAMETER DESCRIPTION  ***
!
!  ALPHA IN  (APPROXIMATE) DIAGONAL ELEMENTS OF THE HESSIAN OF F(X).
!      B IN  ARRAY OF SIMPLE LOWER AND UPPER BOUNDS ON X.  X MUST
!             SATISFY B(1,I) .LE. X(I) .LE. B(2,I), I = 1(1)P.
!             FOR ALL I WITH B(1,I) .GE. B(2,I), DS3GRD SIMPLY
!             SETS G(I) TO 0.
!      D IN  SCALE VECTOR SUCH THAT D(I)*X(I), I = 1,...,P, ARE IN
!             COMPARABLE UNITS.
!   ETA0 IN  ESTIMATED BOUND ON RELATIVE ERROR IN THE FUNCTION VALUE...
!             (TRUE VALUE) = (COMPUTED VALUE)*(1+E),   WHERE
!             ABS(E) .LE. ETA0.
!     FX I/O ON INPUT,  FX  MUST BE THE COMPUTED VALUE OF F(X).  ON
!             OUTPUT WITH IRC = 0, FX HAS BEEN RESTORED TO ITS ORIGINAL
!             VALUE, THE ONE IT HAD WHEN DS3GRD WAS LAST CALLED WITH
!             IRC = 0.
!      G I/O ON INPUT WITH IRC = 0, G SHOULD CONTAIN AN APPROXIMATION
!             TO THE GRADIENT OF F NEAR X, E.G., THE GRADIENT AT THE
!             PREVIOUS ITERATE.  WHEN DS3GRD RETURNS WITH IRC = 0, G IS
!             THE DESIRED FINITE-DIFFERENCE APPROXIMATION TO THE
!             GRADIENT AT X.
!    IRC I/O INPUT/RETURN CODE... BEFORE THE VERY FIRST CALL ON DS3GRD,
!             THE CALLER MUST SET IRC TO 0.  WHENEVER DS3GRD RETURNS A
!             NONZERO VALUE (OF AT MOST P) FOR IRC, IT HAS PERTURBED
!             SOME COMPONENT OF X... THE CALLER SHOULD EVALUATE F(X)
!             AND CALL DS3GRD AGAIN WITH FX = F(X).  IF B PREVENTS
!             ESTIMATING G(I) I.E., IF THERE IS AN I WITH
!             B(1,I) .LT. B(2,I) BUT WITH B(1,I) SO CLOSE TO B(2,I)
!             THAT THE FINITE-DIFFERENCING STEPS CANNOT BE CHOSEN,
!             THEN DS3GRD RETURNS WITH IRC .GT. P.
!      P IN  THE NUMBER OF VARIABLES (COMPONENTS OF X) ON WHICH F
!             DEPENDS.
!      X I/O ON INPUT WITH IRC = 0, X IS THE POINT AT WHICH THE
!             GRADIENT OF F IS DESIRED.  ON OUTPUT WITH IRC NONZERO, X
!             IS THE POINT AT WHICH F SHOULD BE EVALUATED.  ON OUTPUT
!             WITH IRC = 0, X HAS BEEN RESTORED TO ITS ORIGINAL VALUE
!             (THE ONE IT HAD WHEN DS3GRD WAS LAST CALLED WITH IRC = 0)
!             AND G CONTAINS THE DESIRED GRADIENT APPROXIMATION.
!      W I/O WORK VECTOR OF LENGTH 6 IN WHICH DS3GRD SAVES CERTAIN
!             QUANTITIES WHILE THE CALLER IS EVALUATING F(X) AT A
!             PERTURBED X.
!
!     ***  APPLICATION AND USAGE RESTRICTIONS  ***
!
!        THIS ROUTINE IS INTENDED FOR USE WITH QUASI-NEWTON ROUTINES
!     FOR UNCONSTRAINED MINIMIZATION (IN WHICH CASE  ALPHA  COMES FROM
!     THE DIAGONAL OF THE QUASI-NEWTON HESSIAN APPROXIMATION).
!
!     ***  ALGORITHM NOTES  ***
!
!        THIS CODE DEPARTS FROM THE SCHEME PROPOSED BY STEWART (REF. 1)
!     IN ITS GUARDING AGAINST OVERLY LARGE OR SMALL STEP SIZES AND ITS
!     HANDLING OF SPECIAL CASES (SUCH AS ZERO COMPONENTS OF ALPHA OR G).
!
!     ***  REFERENCES  ***
!
! 1. STEWART, G.W. (1967), A MODIFICATION OF DAVIDON*S MINIMIZATION
!        METHOD TO ACCEPT DIFFERENCE APPROXIMATIONS OF DERIVATIVES,
!        J. ASSOC. COMPUT. MACH. 14, PP. 72-83.
!
!     ***  HISTORY  ***
!
!     DESIGNED AND CODED BY DAVID M. GAY (SUMMER 1977/SUMMER 1980).
!
!     ***  GENERAL  ***
!
!        THIS ROUTINE WAS PREPARED IN CONNECTION WITH WORK SUPPORTED BY
!     THE NATIONAL SCIENCE FOUNDATION UNDER GRANTS MCS76-00324 AND
!     MCS-7906671.
!
!.......................................................................
!
!     *****  EXTERNAL FUNCTION  *****
!
      DOUBLE PRECISION DR7MDC
      EXTERNAL DR7MDC
! DR7MDC... RETURNS MACHINE-DEPENDENT CONSTANTS.
!
!     ***** INTRINSIC FUNCTIONS *****
!/+
!      DOUBLE PRECISION DSQRT
!/
!     ***** LOCAL VARIABLES *****
!
      LOGICAL HIT
      INTEGER FH, FX0, HSAVE, I, XISAVE
      DOUBLE PRECISION AAI, AFX, AFXETA, AGI, ALPHAI, AXI, AXIBAR, &
                       DISCON, ETA, GI, H, HMIN, XI, XIH
      DOUBLE PRECISION C2000, FOUR, HMAX0, HMIN0, H0, MACHEP, ONE, P002, &
                       THREE, TWO, ZERO
!
!/6
!     DATA C2000/2.0D+3/, FOUR/4.0D+0/, HMAX0/0.02D+0/, HMIN0/5.0D+1/,
!    1     ONE/1.0D+0/, P002/0.002D+0/, THREE/3.0D+0/,
!    2     TWO/2.0D+0/, ZERO/0.0D+0/
!/7
      PARAMETER (C2000=2.0D+3, FOUR=4.0D+0, HMAX0=0.02D+0, HMIN0=5.0D+1, &
           ONE=1.0D+0, P002=0.002D+0, THREE=3.0D+0, &
           TWO=2.0D+0, ZERO=0.0D+0)
!/
!/6
!     DATA FH/3/, FX0/4/, HSAVE/5/, XISAVE/6/
!/7
      PARAMETER (FH=3, FX0=4, HSAVE=5, XISAVE=6)
!/
!
!---------------------------------  BODY  ------------------------------
!
!      IF (IRC) 80, 10, 210
      IF (IRC .LT. 0) GO TO 80
      IF (IRC .EQ. 0) GO TO 10
      IF (IRC .GT. 0) GO TO 210
!
!     ***  FRESH START -- GET MACHINE-DEPENDENT CONSTANTS  ***
!
!     STORE MACHEP IN W(1) AND H0 IN W(2), WHERE MACHEP IS THE UNIT
!     ROUNDOFF (THE SMALLEST POSITIVE NUMBER SUCH THAT
!     1 + MACHEP .GT. 1  AND  1 - MACHEP .LT. 1),  AND  H0 IS THE
!     SQUARE-ROOT OF MACHEP.
!
 10   W(1) = DR7MDC(3)
      W(2) = SQRT(W(1))
!
      W(FX0) = FX
!
!     ***  INCREMENT  I  AND START COMPUTING  G(I)  ***
!
 20   I = ABS(IRC) + 1
      IF (I .GT. P) GO TO 220
         IRC = I
         IF (B(1,I) .LT. B(2,I)) GO TO 30
            G(I) = ZERO
            GO TO 20
 30      AFX = ABS(W(FX0))
         MACHEP = W(1)
         H0 = W(2)
         HMIN = HMIN0 * MACHEP
         XI = X(I)
         W(XISAVE) = XI
         AXI = ABS(XI)
         AXIBAR = MAX(AXI, ONE/D(I))
         GI = G(I)
         AGI = ABS(GI)
         ETA = ABS(ETA0)
         IF (AFX .GT. ZERO) ETA = MAX(ETA, AGI*AXI*MACHEP/AFX)
         ALPHAI = ALPHA(I)
         IF (ALPHAI .EQ. ZERO) GO TO 130
         IF (GI .EQ. ZERO .OR. FX .EQ. ZERO) GO TO 140
         AFXETA = AFX*ETA
         AAI = ABS(ALPHAI)
!
!        *** COMPUTE H = STEWART*S FORWARD-DIFFERENCE STEP SIZE.
!
         IF (GI**2 .LE. AFXETA*AAI) GO TO 40
              H = TWO*SQRT(AFXETA/AAI)
              H = H*(ONE - AAI*H/(THREE*AAI*H + FOUR*AGI))
              GO TO 50
!40      H = TWO*(AFXETA*AGI/(AAI**2))**(ONE/THREE)
 40      H = TWO * (AFXETA*AGI)**(ONE/THREE) * AAI**(-TWO/THREE)
         H = H*(ONE - TWO*AGI/(THREE*AAI*H + FOUR*AGI))
!
!        ***  ENSURE THAT  H  IS NOT INSIGNIFICANTLY SMALL  ***
!
 50      H = MAX(H, HMIN*AXIBAR)
!
!        *** USE FORWARD DIFFERENCE IF BOUND ON TRUNCATION ERROR IS AT
!        *** MOST 10**-3.
!
         IF (AAI*H .LE. P002*AGI) GO TO 120
!
!        *** COMPUTE H = STEWART*S STEP FOR CENTRAL DIFFERENCE.
!
         DISCON = C2000*AFXETA
         H = DISCON/(AGI + SQRT(GI**2 + AAI*DISCON))
!
!        ***  ENSURE THAT  H  IS NEITHER TOO SMALL NOR TOO BIG  ***
!
         H = MAX(H, HMIN*AXIBAR)
         IF (H .GE. HMAX0*AXIBAR) H = AXIBAR * H0**(TWO/THREE)
!
!        ***  COMPUTE CENTRAL DIFFERENCE  ***
!
         XIH = XI + H
         IF (XI - H .LT. B(1,I)) GO TO 60
         IRC = -I
         IF (XIH .LE. B(2,I)) GO TO 200
            H = -H
            XIH = XI + H
            IF (XI + TWO*H .LT. B(1,I)) GO TO 190
            GO TO 70
 60      IF (XI + TWO*H .GT. B(2,I)) GO TO 190
!        *** MUST DO OFF-SIDE CENTRAL DIFFERENCE ***
 70      IRC = -(I + P)
         GO TO 200
!
 80      I = -IRC
         IF (I .LE. P) GO TO 100
         I = I - P
         IF (I .GT. P) GO TO 90
         W(FH) = FX
         H = TWO * W(HSAVE)
         XIH = W(XISAVE) + H
         IRC = IRC - P
         GO TO 200
!
!    *** FINISH OFF-SIDE CENTRAL DIFFERENCE ***
!
 90      I = I - P
         G(I) = (FOUR*W(FH) - FX - THREE*W(FX0)) / W(HSAVE)
         IRC = I
         X(I) = W(XISAVE)
         GO TO 20
!
 100     H = -W(HSAVE)
         IF (H .GT. ZERO) GO TO 110
         W(FH) = FX
         XIH = W(XISAVE) + H
         GO TO 200
!
 110     G(I) = (W(FH) - FX) / (TWO * H)
         X(I) = W(XISAVE)
         GO TO 20
!
!     ***  COMPUTE FORWARD DIFFERENCES IN VARIOUS CASES  ***
!
 120     IF (H .GE. HMAX0*AXIBAR) H = H0 * AXIBAR
         IF (ALPHAI*GI .LT. ZERO) H = -H
         GO TO 150
 130     H = AXIBAR
         GO TO 150
 140     H = H0 * AXIBAR
!
 150     HIT = .FALSE.
 160     XIH = XI + H
         IF (H .GT. ZERO) GO TO 170
            IF (XIH .GE. B(1,I)) GO TO 200
            GO TO 180
 170     IF (XIH .LE. B(2,I)) GO TO 200
 180        IF (HIT) GO TO 190
            HIT = .TRUE.
            H = -H
            GO TO 160
!
!        *** ERROR RETURN...
 190     IRC = I + P
         GO TO 230
!
!        *** RETURN FOR NEW FUNCTION VALUE...
 200     X(I) = XIH
         W(HSAVE) = H
         GO TO 999
!
!     ***  COMPUTE ACTUAL FORWARD DIFFERENCE  ***
!
 210     G(IRC) = (FX - W(FX0)) / W(HSAVE)
         X(IRC) = W(XISAVE)
         GO TO 20
!
!  ***  RESTORE FX AND INDICATE THAT G HAS BEEN COMPUTED  ***
!
 220  IRC = 0
 230  FX = W(FX0)
!
 999  RETURN
!  ***  LAST LINE OF DS3GRD FOLLOWS  ***
      END
      SUBROUTINE DV2AXY(P, W, A, X, Y)
!
!  ***  SET W = A*X + Y  --  W, X, Y = P-VECTORS, A = SCALAR  ***
!
      INTEGER P
      DOUBLE PRECISION A, W(P), X(P), Y(P)
!
      INTEGER I
!
      DO I = 1, P
 10      W(I) = A*X(I) + Y(I)
      END DO
      RETURN
      END
      DOUBLE PRECISION FUNCTION DV2NRM(P, X)
!
!  ***  RETURN THE 2-NORM OF THE P-VECTOR X, TAKING  ***
!  ***  CARE TO AVOID THE MOST LIKELY UNDERFLOWS.    ***
!
      INTEGER P
      DOUBLE PRECISION X(P)
!
      INTEGER I, J
      DOUBLE PRECISION ONE, R, SCALE, SQTETA, T, XI, ZERO
!/+
!      DOUBLE PRECISION DSQRT
!/
      DOUBLE PRECISION DR7MDC
      EXTERNAL DR7MDC
!
!/6
!     DATA ONE/1.D+0/, ZERO/0.D+0/
!/7
      PARAMETER (ONE=1.D+0, ZERO=0.D+0)
      SAVE SQTETA
!/
      DATA SQTETA/0.D+0/
!
      IF (P .GT. 0) GO TO 10
         DV2NRM = ZERO
         GO TO 999
 10   DO I = 1, P
         IF (X(I) .NE. ZERO) GO TO 30
 20      CONTINUE
      END DO
      DV2NRM = ZERO
      GO TO 999
!
 30   SCALE = ABS(X(I))
      IF (I .LT. P) GO TO 40
         DV2NRM = SCALE
         GO TO 999
 40   T = ONE
      IF (SQTETA .EQ. ZERO) SQTETA = DR7MDC(2)
!
!     ***  SQTETA IS (SLIGHTLY LARGER THAN) THE SQUARE ROOT OF THE
!     ***  SMALLEST POSITIVE FLOATING POINT NUMBER ON THE MACHINE.
!     ***  THE TESTS INVOLVING SQTETA ARE DONE TO PREVENT UNDERFLOWS.
!
      J = I + 1
      DO I = J, P
         XI = ABS(X(I))
         IF (XI .GT. SCALE) GO TO 50
              R = XI / SCALE
              IF (R .GT. SQTETA) T = T + R*R
              CYCLE
 50           R = SCALE / XI
              IF (R .LE. SQTETA) R = ZERO
              T = ONE  +  T * R*R
              SCALE = XI
 60      CONTINUE
      END DO
!
      DV2NRM = SCALE * SQRT(T)
 999  RETURN
!  ***  LAST LINE OF DV2NRM FOLLOWS  ***
      END
      SUBROUTINE DV7CPY(P, Y, X)
!
!  ***  SET Y = X, WHERE X AND Y ARE P-VECTORS  ***
!
      INTEGER P
      DOUBLE PRECISION X(P), Y(P)
!
      INTEGER I
!
      DO I = 1, P
 10      Y(I) = X(I)
      END DO
      RETURN
      END
      SUBROUTINE DV7DFL(ALG, LV, V)
!
!  ***  SUPPLY ***SOL (VERSION 2.3) DEFAULT VALUES TO V  ***
!
!  ***  ALG = 1 MEANS REGRESSION CONSTANTS.
!  ***  ALG = 2 MEANS GENERAL UNCONSTRAINED OPTIMIZATION CONSTANTS.
!
      INTEGER ALG, LV
      DOUBLE PRECISION V(LV)
!
      DOUBLE PRECISION DR7MDC
      EXTERNAL DR7MDC
! DR7MDC... RETURNS MACHINE-DEPENDENT CONSTANTS
!
      DOUBLE PRECISION MACHEP, MEPCRT, ONE, SQTEPS, THREE
!
!  ***  SUBSCRIPTS FOR V  ***
!
      INTEGER AFCTOL, BIAS, COSMIN, DECFAC, DELTA0, DFAC, DINIT, DLTFDC, &
              DLTFDJ, DTINIT, D0INIT, EPSLON, ETA0, FUZZ, HUBERC, &
              INCFAC, LMAX0, LMAXS, PHMNFC, PHMXFC, RDFCMN, RDFCMX, &
              RFCTOL, RLIMIT, RSPTOL, SCTOL, SIGMIN, TUNER1, TUNER2, &
              TUNER3, TUNER4, TUNER5, XCTOL, XFTOL
!
!/6
!     DATA ONE/1.D+0/, THREE/3.D+0/
!/7
      PARAMETER (ONE=1.D+0, THREE=3.D+0)
!/
!
!  ***  V SUBSCRIPT VALUES  ***
!
!/6
!     DATA AFCTOL/31/, BIAS/43/, COSMIN/47/, DECFAC/22/, DELTA0/44/,
!    1     DFAC/41/, DINIT/38/, DLTFDC/42/, DLTFDJ/43/, DTINIT/39/,
!    2     D0INIT/40/, EPSLON/19/, ETA0/42/, FUZZ/45/, HUBERC/48/,
!    3     INCFAC/23/, LMAX0/35/, LMAXS/36/, PHMNFC/20/, PHMXFC/21/,
!    4     RDFCMN/24/, RDFCMX/25/, RFCTOL/32/, RLIMIT/46/, RSPTOL/49/,
!    5     SCTOL/37/, SIGMIN/50/, TUNER1/26/, TUNER2/27/, TUNER3/28/,
!    6     TUNER4/29/, TUNER5/30/, XCTOL/33/, XFTOL/34/
!/7
      PARAMETER (AFCTOL=31, BIAS=43, COSMIN=47, DECFAC=22, DELTA0=44, &
                 DFAC=41, DINIT=38, DLTFDC=42, DLTFDJ=43, DTINIT=39, &
                 D0INIT=40, EPSLON=19, ETA0=42, FUZZ=45, HUBERC=48, &
                 INCFAC=23, LMAX0=35, LMAXS=36, PHMNFC=20, PHMXFC=21, &
                 RDFCMN=24, RDFCMX=25, RFCTOL=32, RLIMIT=46, RSPTOL=49, &
                 SCTOL=37, SIGMIN=50, TUNER1=26, TUNER2=27, TUNER3=28, &
                 TUNER4=29, TUNER5=30, XCTOL=33, XFTOL=34)
!/
!
!-------------------------------  BODY  --------------------------------
!
      MACHEP = DR7MDC(3)
      V(AFCTOL) = 1.D-20
      IF (MACHEP .GT. 1.D-10) V(AFCTOL) = MACHEP**2
      V(DECFAC) = 0.5D+0
      SQTEPS = DR7MDC(4)
      V(DFAC) = 0.6D+0
      V(DTINIT) = 1.D-6
      MEPCRT = MACHEP ** (ONE/THREE)
      V(D0INIT) = 1.D+0
      V(EPSLON) = 0.1D+0
      V(INCFAC) = 2.D+0
      V(LMAX0) = 1.D+0
      V(LMAXS) = 1.D+0
      V(PHMNFC) = -0.1D+0
      V(PHMXFC) = 0.1D+0
      V(RDFCMN) = 0.1D+0
      V(RDFCMX) = 4.D+0
      V(RFCTOL) = MAX(1.D-10, MEPCRT**2)
      V(SCTOL) = V(RFCTOL)
      V(TUNER1) = 0.1D+0
      V(TUNER2) = 1.D-4
      V(TUNER3) = 0.75D+0
      V(TUNER4) = 0.5D+0
      V(TUNER5) = 0.75D+0
      V(XCTOL) = SQTEPS
      V(XFTOL) = 1.D+2 * MACHEP
!
      IF (ALG .GE. 2) GO TO 10
!
!  ***  REGRESSION  VALUES
!
      V(COSMIN) = MAX(1.D-6, 1.D+2 * MACHEP)
      V(DINIT) = 0.D+0
      V(DELTA0) = SQTEPS
      V(DLTFDC) = MEPCRT
      V(DLTFDJ) = SQTEPS
      V(FUZZ) = 1.5D+0
      V(HUBERC) = 0.7D+0
      V(RLIMIT) = DR7MDC(5)
      V(RSPTOL) = 1.D-3
      V(SIGMIN) = 1.D-4
      GO TO 999
!
!  ***  GENERAL OPTIMIZATION VALUES
!
 10   V(BIAS) = 0.8D+0
      V(DINIT) = -1.0D+0
      V(ETA0) = 1.0D+3 * MACHEP
!
 999  RETURN
!  ***  LAST CARD OF DV7DFL FOLLOWS  ***
      END
      SUBROUTINE DV7IPR(N, IP, X)
!
!     PERMUTE X SO THAT X.OUTPUT(I) = X.INPUT(IP(I)).
!     IP IS UNCHANGED ON OUTPUT.
!
      INTEGER N
      INTEGER IP(N)
      DOUBLE PRECISION X(N)
!
      INTEGER I, J, K
      DOUBLE PRECISION T
      DO I = 1, N
         J = IP(I)
         IF (J .EQ. I) CYCLE 
         IF (J .GT. 0) GO TO 10
            IP(I) = -J
            CYCLE 
 10      T = X(I)
         K = I
 20      X(K) = X(J)
         K = J
         J = IP(K)
         IP(K) = -J
         IF (J .GT. I) GO TO 20
         X(K) = T
 30      CONTINUE
      END DO
 999  RETURN
!  ***  LAST LINE OF DV7IPR FOLLOWS  ***
      END
      SUBROUTINE DV7SCP(P, Y, S)
!
!  ***  SET P-VECTOR Y TO SCALAR S  ***
!
      INTEGER P
      DOUBLE PRECISION S, Y(P)
!
      INTEGER I
!
      DO I = 1, P
 10      Y(I) = S
      END DO
      RETURN
      END
      SUBROUTINE DV7SHF(N, K, X)
!
!  ***  SHIFT X(K),...,X(N) LEFT CIRCULARLY ONE POSITION  ***
!
      INTEGER N, K
      DOUBLE PRECISION X(N)
!
      INTEGER I, NM1
      DOUBLE PRECISION T
!
      IF (K .GE. N) GO TO 999
      NM1 = N - 1
      T = X(K)
      DO I = K, NM1
 10      X(I) = X(I+1)
      END DO
      X(N) = T
 999  RETURN
      END
      SUBROUTINE DV7VMP(N, X, Y, Z, K)
!
! ***  SET X(I) = Y(I) * Z(I)**K, 1 .LE. I .LE. N (FOR K = 1 OR -1)  ***
!
      INTEGER N, K
      DOUBLE PRECISION X(N), Y(N), Z(N)
      INTEGER I
!
      IF (K .GE. 0) GO TO 20
      DO I = 1, N
 10      X(I) = Y(I) / Z(I)
      END DO
      GO TO 999
!
 20   DO I = 1, N
 30      X(I) = Y(I) * Z(I)
      END DO
 999  RETURN
!  ***  LAST CARD OF DV7VMP FOLLOWS  ***
      END
      SUBROUTINE DW7ZBF (L, N, S, W, Y, Z)
!
!  ***  COMPUTE  Y  AND  Z  FOR  DL7UPD  CORRESPONDING TO BFGS UPDATE.
!
      INTEGER N
      DOUBLE PRECISION L(1), S(N), W(N), Y(N), Z(N)
!     DIMENSION L(N*(N+1)/2)
!
!--------------------------  PARAMETER USAGE  --------------------------
!
! L (I/O) CHOLESKY FACTOR OF HESSIAN, A LOWER TRIANG. MATRIX STORED
!             COMPACTLY BY ROWS.
! N (INPUT) ORDER OF  L  AND LENGTH OF  S,  W,  Y,  Z.
! S (INPUT) THE STEP JUST TAKEN.
! W (OUTPUT) RIGHT SINGULAR VECTOR OF RANK 1 CORRECTION TO L.
! Y (INPUT) CHANGE IN GRADIENTS CORRESPONDING TO S.
! Z (OUTPUT) LEFT SINGULAR VECTOR OF RANK 1 CORRECTION TO L.
!
!-------------------------------  NOTES  -------------------------------
!
!  ***  ALGORITHM NOTES  ***
!
!        WHEN  S  IS COMPUTED IN CERTAIN WAYS, E.G. BY  GQTSTP  OR
!     DBLDOG,  IT IS POSSIBLE TO SAVE N**2/2 OPERATIONS SINCE  (L**T)*S
!     OR  L*(L**T)*S IS THEN KNOWN.
!        IF THE BFGS UPDATE TO L*(L**T) WOULD REDUCE ITS DETERMINANT TO
!     LESS THAN EPS TIMES ITS OLD VALUE, THEN THIS ROUTINE IN EFFECT
!     REPLACES  Y  BY  THETA*Y + (1 - THETA)*L*(L**T)*S,  WHERE  THETA
!     (BETWEEN 0 AND 1) IS CHOSEN TO MAKE THE REDUCTION FACTOR = EPS.
!
!  ***  GENERAL  ***
!
!     CODED BY DAVID M. GAY (FALL 1979).
!     THIS SUBROUTINE WAS WRITTEN IN CONNECTION WITH RESEARCH SUPPORTED
!     BY THE NATIONAL SCIENCE FOUNDATION UNDER GRANTS MCS-7600324 AND
!     MCS-7906671.
!
!------------------------  EXTERNAL QUANTITIES  ------------------------
!
!  ***  FUNCTIONS AND SUBROUTINES CALLED  ***
!
      DOUBLE PRECISION DD7TPR
      EXTERNAL DD7TPR, DL7IVM, DL7TVM
! DD7TPR RETURNS INNER PRODUCT OF TWO VECTORS.
! DL7IVM MULTIPLIES L**-1 TIMES A VECTOR.
! DL7TVM MULTIPLIES L**T TIMES A VECTOR.
!
!  ***  INTRINSIC FUNCTIONS  ***
!/+
!      DOUBLE PRECISION DSQRT
!/
!--------------------------  LOCAL VARIABLES  --------------------------
!
      INTEGER I
      DOUBLE PRECISION CS, CY, EPS, EPSRT, ONE, SHS, YS, THETA
!
!  ***  DATA INITIALIZATIONS  ***
!
!/6
!     DATA EPS/0.1D+0/, ONE/1.D+0/
!/7
      PARAMETER (EPS=0.1D+0, ONE=1.D+0)
!/
!
!+++++++++++++++++++++++++++++++  BODY  ++++++++++++++++++++++++++++++++
!
      CALL DL7TVM(N, W, L, S)
      SHS = DD7TPR(N, W, W)
      YS = DD7TPR(N, Y, S)
      IF (YS .GE. EPS*SHS) GO TO 10
         THETA = (ONE - EPS) * SHS / (SHS - YS)
         EPSRT = SQRT(EPS)
         CY = THETA / (SHS * EPSRT)
         CS = (ONE + (THETA-ONE)/EPSRT) / SHS
         GO TO 20
 10   CY = ONE / (SQRT(YS) * SQRT(SHS))
      CS = ONE / SHS
 20   CALL DL7IVM(N, Z, L, Y)
      DO I = 1, N
 30      Z(I) = CY * Z(I)  -  CS * W(I)
      END DO
!
 999  RETURN
!  ***  LAST CARD OF DW7ZBF FOLLOWS  ***
      END
!DECK I1MACH
      INTEGER FUNCTION I1MACH (I)
      IMPLICIT NONE
      INTEGER :: I
      REAL :: X
      DOUBLE PRECISION :: XX
!***BEGIN PROLOGUE  I1MACH
!***PURPOSE  Return integer machine dependent constants.
!***LIBRARY   SLATEC
!***CATEGORY  R1
!***TYPE      INTEGER (I1MACH-I)
!***KEYWORDS  MACHINE CONSTANTS
!***AUTHOR  Fox, P. A., (Bell Labs)
!           Hall, A. D., (Bell Labs)
!           Schryer, N. L., (Bell Labs)
!***DESCRIPTION
!
!   I1MACH can be used to obtain machine-dependent parameters for the
!   local machine environment.  It is a function subprogram with one
!   (input) argument and can be referenced as follows:
!
!        K = I1MACH(I)
!
!   where I=1,...,16.  The (output) value of K above is determined by
!   the (input) value of I.  The results for various values of I are
!   discussed below.
!
!   I/O unit numbers:
!     I1MACH( 1) = the standard input unit.
!     I1MACH( 2) = the standard output unit.
!     I1MACH( 3) = the standard punch unit.
!     I1MACH( 4) = the standard error message unit.
!
!   Words:
!     I1MACH( 5) = the number of bits per integer storage unit.
!     I1MACH( 6) = the number of characters per integer storage unit.
!
!   Integers:
!     assume integers are represented in the S-digit, base-A form
!
!                sign ( X(S-1)*A**(S-1) + ... + X(1)*A + X(0) )
!
!                where 0 .LE. X(I) .LT. A for I=0,...,S-1.
!     I1MACH( 7) = A, the base.
!     I1MACH( 8) = S, the number of base-A digits.
!     I1MACH( 9) = A**S - 1, the largest magnitude.
!
!   Floating-Point Numbers:
!     Assume floating-point numbers are represented in the T-digit,
!     base-B form
!                sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
!
!                where 0 .LE. X(I) .LT. B for I=1,...,T,
!                0 .LT. X(1), and EMIN .LE. E .LE. EMAX.
!     I1MACH(10) = B, the base.
!
!   Single-Precision:
!     I1MACH(11) = T, the number of base-B digits.
!     I1MACH(12) = EMIN, the smallest exponent E.
!     I1MACH(13) = EMAX, the largest exponent E.
!
!   Double-Precision:
!     I1MACH(14) = T, the number of base-B digits.
!     I1MACH(15) = EMIN, the smallest exponent E.
!     I1MACH(16) = EMAX, the largest exponent E.
!
!   To alter this function for a particular environment, the desired
!   set of DATA statements should be activated by removing the C from
!   column 1.  Also, the values of I1MACH(1) - I1MACH(4) should be
!   checked for consistency with the local operating system.
!
!***REFERENCES  P. A. Fox, A. D. Hall and N. L. Schryer, Framework for
!                 a portable library, ACM Transactions on Mathematical
!                 Software 4, 2 (June 1978), pp. 177-188.
!***ROUTINES CALLED  (NONE)
!***REVISION HISTORY  (YYMMDD)
!   750101  DATE WRITTEN
!   960411  Modified for Fortran 90 (BE after suggestions by EHG).   
!   980727  Modified value of I1MACH(6) (BE after suggestion by EHG).   
!***END PROLOGUE  I1MACH
!
      X  = 1.0      
      XX = 1.0D0

      SELECT CASE (I)
        CASE (1)
          I1MACH = 5 ! Input unit
        CASE (2)
          I1MACH = 6 ! Output unit
        CASE (3)
          I1MACH = 0 ! Punch unit is no longer used
        CASE (4)
          I1MACH = 0 ! Error message unit
        CASE (5)
          I1MACH = BIT_SIZE(I)
        CASE (6)
          I1MACH = 4            ! Characters per integer is hopefully no
                                ! longer used. 
                                ! If it is used it has to be set manually.
                                ! The value 4 is correct on IEEE-machines.
        CASE (7)
          I1MACH = RADIX(1)
        CASE (8)
          I1MACH = BIT_SIZE(I) - 1
        CASE (9)
          I1MACH = HUGE(1)
        CASE (10)
          I1MACH = RADIX(X)
        CASE (11)
          I1MACH = DIGITS(X)
        CASE (12)
          I1MACH = MINEXPONENT(X)
        CASE (13)
          I1MACH = MAXEXPONENT(X)
        CASE (14)
          I1MACH = DIGITS(XX)
        CASE (15)
          I1MACH = MINEXPONENT(XX)
        CASE (16)
          I1MACH = MAXEXPONENT(XX) 
        CASE DEFAULT
          WRITE (*, FMT = 9000)
 9000     FORMAT ('1ERROR    1 IN I1MACH - I OUT OF BOUNDS')
          STOP
        END SELECT
      RETURN
      END
      INTEGER FUNCTION I7MDCN(K)
!
      INTEGER K
!
!  ***  RETURN INTEGER MACHINE-DEPENDENT CONSTANTS  ***
!
!     ***  K = 1 MEANS RETURN STANDARD OUTPUT UNIT NUMBER.   ***
!     ***  K = 2 MEANS RETURN ALTERNATE OUTPUT UNIT NUMBER.  ***
!     ***  K = 3 MEANS RETURN  INPUT UNIT NUMBER.            ***
!          (NOTE -- K = 2, 3 ARE USED ONLY BY TEST PROGRAMS.)
!
!  +++  PORT VERSION FOLLOWS...
      INTEGER I1MACH
      EXTERNAL I1MACH
      INTEGER MDPERM(3)
      DATA MDPERM(1)/2/, MDPERM(2)/4/, MDPERM(3)/1/
      I7MDCN = I1MACH(MDPERM(K))
!  +++  END OF PORT VERSION  +++
!
!  +++  NON-PORT VERSION FOLLOWS...
!     INTEGER MDCON(3)
!     DATA MDCON(1)/6/, MDCON(2)/8/, MDCON(3)/5/
!     I7MDCN = MDCON(K)
!  +++  END OF NON-PORT VERSION  +++
!
 999  RETURN
!  ***  LAST CARD OF I7MDCN FOLLOWS  ***
      END
      SUBROUTINE I7SHFT(N, K, X)
!
!  ***  SHIFT X(K),...,X(N) LEFT CIRCULARLY ONE POSITION IF K .GT. 0.
!  ***  SHIFT X(-K),...,X(N) RIGHT CIRCULARLY ONE POSITION IF K .LT. 0.
!
      INTEGER N, K
      INTEGER X(N)
!
      INTEGER I, II, K1, NM1, T
!
      IF (K .LT. 0) GO TO 20
      IF (K .GE. N) GO TO 999
      NM1 = N - 1
      T = X(K)
      DO I = K, NM1
 10      X(I) = X(I+1)
      END DO
      X(N) = T
      GO TO 999
!
 20   K1 = -K
      IF (K1 .GE. N) GO TO 999
      T = X(N)
      NM1 = N - K1
      DO II = 1, NM1
         I = N - II
         X(I+1) = X(I)
 30      CONTINUE
      END DO
      X(K1) = T
 999  RETURN
!  ***  LAST LINE OF I7SHFT FOLLOWS  ***
      END
      LOGICAL FUNCTION STOPX(IDUMMY)
!     *****PARAMETERS...
      INTEGER IDUMMY
!
!     ..................................................................
!
!     *****PURPOSE...
!     THIS FUNCTION MAY SERVE AS THE STOPX (ASYNCHRONOUS INTERRUPTION)
!     FUNCTION FOR THE NL2SOL (NONLINEAR LEAST-SQUARES) PACKAGE AT
!     THOSE INSTALLATIONS WHICH DO NOT WISH TO IMPLEMENT A
!     DYNAMIC STOPX.
!
!     *****ALGORITHM NOTES...
!     AT INSTALLATIONS WHERE THE NL2SOL SYSTEM IS USED
!     INTERACTIVELY, THIS DUMMY STOPX SHOULD BE REPLACED BY A
!     FUNCTION THAT RETURNS .TRUE. IF AND ONLY IF THE INTERRUPT
!     (BREAK) KEY HAS BEEN PRESSED SINCE THE LAST CALL ON STOPX.
!
!     ..................................................................
!
      STOPX = .FALSE.
      RETURN
      END
!
      DOUBLE PRECISION FUNCTION D1MACH(I)
      INTEGER :: I
      DOUBLE PRECISION :: B, X = 1.D0

      B = RADIX(X)

      SELECT CASE (I)
        CASE (1)
          D1MACH = TINY(X)            ! smallest positive magnitude.
        CASE (2)
          D1MACH = HUGE(X)            ! largest magnitude.
        CASE (3)
          D1MACH = B**(-DIGITS(X))    ! smallest relative spacing.
        CASE (4)
          D1MACH = B**(1-DIGITS(X))   ! largest relative spacing.
        CASE (5)
          D1MACH = LOG10(B)
        CASE DEFAULT
          STOP 'D1MACH -- input arg out of bounds'
      END SELECT
      RETURN
!
      END
